@@ -88,7 +88,7 @@ static void locate_unmanaged_files(
     for(i=0; i<argc; i++){
       file_canonical_name(argv[i], &name, 0);
       zName = blob_str(&name);
-      isDir = file_wd_isdir(zName);
+      isDir = file_isdir(zName, RepoFILE);
       if( isDir==1 ){
         vfile_scan(&name, nRoot-1, scanFlags, pIgnore, 0);
       }else if( isDir==0 ){
@@ -211,7 +211,7 @@ static void status_report(
     int isNew = isManaged && !db_column_int(&q, 5);
     int isRenamed = db_column_int(&q, 6);
     char *zFullName = mprintf("%s%s", g.zLocalRoot, zPathname);
-    int isMissing = !file_wd_isfile_or_link(zFullName);
+    int isMissing = !file_isfile_or_link(zFullName);
 
     /* Determine the file change classification, if any. */
     if( isDeleted ){
@@ -256,7 +256,7 @@ static void status_report(
       zClass = "UNEXEC";
     }else if( (flags & C_META) && isChnged==9 ){
       zClass = "UNLINK";
-    }else if( (flags & C_CONFLICT) && isChnged && !file_wd_islink(zFullName)
+    }else if( (flags & C_CONFLICT) && isChnged && !file_islink(zFullName)
            && file_contains_merge_marker(zFullName) ){
       zClass = "CONFLICT";
     }else if( (flags & (C_EDITED | C_CHANGED)) && isChnged
@@ -309,8 +309,7 @@ static void status_report(
   /* If C_MERGE, put merge contributors at the end of the report. */
 skipFiles:
   if( flags & C_MERGE ){
-    db_prepare(&q, "SELECT uuid, id FROM vmerge JOIN blob ON merge=rid"
-                   " WHERE id<=0");
+    db_prepare(&q, "SELECT mhash, id FROM vmerge WHERE id<=0" );
     while( db_step(&q)==SQLITE_ROW ){
       if( flags & C_COMMENT ){
         blob_append(report, "# ", 2);
@@ -419,7 +418,6 @@ static int determine_cwd_relative_option()
 **    --case-sensitive <BOOL>  Override case-sensitive setting.
 **    --dotfiles        Include unmanaged files beginning with a dot.
 **    --ignore <CSG>    Ignore unmanaged files matching CSG glob patterns.
-**    --no-dir-symlinks Disables support for directory symlinks.
 **
 ** Options specific to the changes command:
 **    --header          Identify the repository if report is non-empty.
@@ -475,6 +473,8 @@ void status_cmd(void){
   unsigned flags = 0;
   int vid, i;
 
+  fossil_pledge("stdio rpath wpath cpath fattr id flock tty chown");
+
   /* Load affirmative flag options. */
   for( i=0; i<count(flagDefs); ++i ){
     if( (command==CHANGES || !(flagDefs[i].mask & C_CLASSIFY))
@@ -558,8 +558,9 @@ void status_cmd(void){
   status_report(&report, flags);
   if( blob_size(&report) ){
     if( showHdr ){
-      fossil_print("Changes for %s at %s:\n", db_get("project-name", "???"),
-                   g.zLocalRoot);
+      fossil_print(
+        "Changes for %s at %s:\n", db_get("project-name", "<unnamed>"),
+        g.zLocalRoot);
     }
     blob_write_to_file(&report, "-");
   }else if( verboseFlag ){
@@ -767,7 +768,7 @@ void ls_cmd(void){
         type = "ADDED      ";
       }else if( isDeleted ){
         type = "DELETED    ";
-      }else if( !file_wd_isfile_or_link(zFullName) ){
+      }else if( !file_isfile_or_link(zFullName) ){
         if( file_access(zFullName, F_OK)==0 ){
           type = "NOT_A_FILE ";
         }else{
@@ -828,7 +829,6 @@ void ls_cmd(void){
 **    --dotfiles       include files beginning with a dot (".")
 **    --header         Identify the repository if there are extras
 **    --ignore <CSG>   ignore files matching patterns from the argument
-**    --no-dir-symlinks Disables support for directory symlinks.
 **    --rel-paths      Display pathnames relative to the current working
 **                     directory.
 **
@@ -858,6 +858,8 @@ void extras_cmd(void){
     zIgnoreFlag = db_get("ignore-glob", 0);
   }
   pIgnore = glob_create(zIgnoreFlag);
+  /* Always consider symlinks. */
+  g.allowSymlinks = db_allow_symlinks_by_default();
   locate_unmanaged_files(g.argc-2, g.argv+2, scanFlags, pIgnore);
   glob_free(pIgnore);
 
@@ -865,7 +867,7 @@ void extras_cmd(void){
   status_report(&report, flags);
   if( blob_size(&report) ){
     if( showHdr ){
-      fossil_print("Extras for %s at %s:\n", db_get("project-name","???"),
+      fossil_print("Extras for %s at %s:\n", db_get("project-name","<unnamed>"),
                    g.zLocalRoot);
     }
     blob_write_to_file(&report, "-");
@@ -903,10 +905,9 @@ void extras_cmd(void){
 ** the (versionable) clean-glob, ignore-glob, and keep-glob settings.
 **
 ** The --verily option ignores the keep-glob and ignore-glob settings and
-** turns on the options --force, --emptydirs, --dotfiles, --disable-undo,
-** and --no-dir-symlinks.  Use the --verily option when you really want
-** to clean up everything.  Extreme care should be exercised when using
-** the --verily option.
+** turns on --force, --emptydirs, --dotfiles, and --disable-undo.  Use the
+** --verily option when you really want to clean up everything.  Extreme
+** care should be exercised when using the --verily option.
 **
 ** Options:
 **    --allckouts      Check for empty directories within any checkouts
@@ -936,7 +937,7 @@ void extras_cmd(void){
 **    -x|--verily      WARNING: Removes everything that is not a managed
 **                     file or the repository itself.  This option
 **                     implies the --force, --emptydirs, --dotfiles, and
-**                     --disable-undo, and --no-dir-symlinks options.
+**                     --disable-undo options.
 **                     Furthermore, it completely disregards the keep-glob
 **                     and ignore-glob settings.  However, it does honor
 **                     the --ignore and --keep options.
@@ -952,7 +953,6 @@ void extras_cmd(void){
 **                     deleted.
 **    --no-prompt      This option disables prompting the user for input
 **                     and assumes an answer of 'No' for every question.
-**    --no-dir-symlinks Disables support for directory symlinks.
 **    --temp           Remove only Fossil-generated temporary files.
 **    -v|--verbose     Show all files as they are removed.
 **
@@ -1001,8 +1001,6 @@ void clean_cmd(void){
     disableUndo = 1;
     scanFlags |= SCAN_ALL;
     zCleanFlag = 0;
-    g.fNoDirSymlinks = 1; /* Forbid symlink directory traversal. */
-    g.allowSymlinks = 1;  /* Treat symlink files as content. */
   }
   if( zIgnoreFlag==0 && !verilyFlag ){
     zIgnoreFlag = db_get("ignore-glob", 0);
@@ -1019,6 +1017,8 @@ void clean_cmd(void){
   pKeep = glob_create(zKeepFlag);
   pClean = glob_create(zCleanFlag);
   nRoot = (int)strlen(g.zLocalRoot);
+  /* Always consider symlinks. */
+  g.allowSymlinks = db_allow_symlinks_by_default();
   if( !dirsOnlyFlag ){
     Stmt q;
     Blob repo;
@@ -1193,13 +1193,15 @@ void prompt_for_user_comment(Blob *pComment, Blob *pPrompt){
   }
 #endif
   if( zEditor==0 ){
-    blob_append(pPrompt,
-       "#\n"
-       "# Since no default text editor is set using EDITOR or VISUAL\n"
-       "# environment variables or the \"fossil set editor\" command,\n"
-       "# and because no comment was specified using the \"-m\" or \"-M\"\n"
-       "# command-line options, you will need to enter the comment below.\n"
-       "# Type \".\" on a line by itself when you are done:\n", -1);
+    if( blob_size(pPrompt)>0 ){
+      blob_append(pPrompt,
+         "#\n"
+         "# Since no default text editor is set using EDITOR or VISUAL\n"
+         "# environment variables or the \"fossil set editor\" command,\n"
+         "# and because no comment was specified using the \"-m\" or \"-M\"\n"
+         "# command-line options, you will need to enter the comment below.\n"
+         "# Type \".\" on a line by itself when you are done:\n", -1);
+    }
     zFile = mprintf("-");
   }else{
     Blob fname;
@@ -1209,7 +1211,7 @@ void prompt_for_user_comment(Blob *pComment, Blob *pPrompt){
       zFile = db_text(0, "SELECT '%qci-comment-'||hex(randomblob(6))||'.txt'",
                       blob_str(&fname));
     }else{
-      file_tempname(&fname, "ci-comment");
+      file_tempname(&fname, "ci-comment",0);
       zFile = mprintf("%s", blob_str(&fname));
     }
     blob_reset(&fname);
@@ -1217,7 +1219,7 @@ void prompt_for_user_comment(Blob *pComment, Blob *pPrompt){
 #if defined(_WIN32)
   blob_add_cr(pPrompt);
 #endif
-  blob_write_to_file(pPrompt, zFile);
+  if( blob_size(pPrompt)>0 ) blob_write_to_file(pPrompt, zFile);
   if( zEditor ){
     zCmd = mprintf("%s \"%s\"", zEditor, zFile);
     fossil_print("%s\n", zCmd);
@@ -1225,7 +1227,7 @@ void prompt_for_user_comment(Blob *pComment, Blob *pPrompt){
       fossil_fatal("editor aborted: \"%s\"", zCmd);
     }
 
-    blob_read_from_file(&reply, zFile);
+    blob_read_from_file(&reply, zFile, ExtFILE);
   }else{
     char zIn[300];
     blob_zero(&reply);
@@ -1566,7 +1568,7 @@ static void create_manifest(
     if( isSelected ){
       int mPerm;
 
-      mPerm = file_wd_perm(blob_str(&filename));
+      mPerm = file_perm(blob_str(&filename), RepoFILE);
       isExe = ( mPerm==PERM_EXE );
       isLink = ( mPerm==PERM_LNK );
     }
@@ -1634,10 +1636,9 @@ static void create_manifest(
   free(zDate);
 
   db_prepare(&q,
-    "SELECT CASE vmerge.id WHEN -1 THEN '+' ELSE '-' END || blob.uuid, merge"
-    "  FROM vmerge, blob"
+    "SELECT CASE vmerge.id WHEN -1 THEN '+' ELSE '-' END || mhash, merge"
+    "  FROM vmerge"
     " WHERE (vmerge.id=-1 OR vmerge.id=-2)"
-    "   AND blob.rid=vmerge.merge"
     " ORDER BY 1");
   while( db_step(&q)==SQLITE_ROW ){
     const char *zCherrypickUuid = db_column_text(&q, 0);
@@ -1666,7 +1667,7 @@ static void create_manifest(
   if( p->closeFlag ){
     blob_appendf(pOut, "T +closed *\n");
   }
-  db_prepare(&q, "SELECT uuid,merge FROM vmerge JOIN blob ON merge=rid"
+  db_prepare(&q, "SELECT mhash,merge FROM vmerge"
                  " WHERE id %s ORDER BY 1",
                  p->integrateFlag ? "IN(0,-4)" : "=(-4)");
   while( db_step(&q)==SQLITE_ROW ){
@@ -1912,11 +1913,7 @@ void test_commit_warning(void){
     binOk = db_column_int(&q, 3);
     encodingOk = db_column_int(&q, 4);
     blob_zero(&content);
-    if( file_wd_islink(zFullname) ){
-      blob_read_link(&content, zFullname);
-    }else{
-      blob_read_from_file(&content, zFullname);
-    }
+    blob_read_from_file(&content, zFullname, RepoFILE);
     blob_zero(&reason);
     fileRc = commit_warning(&content, crlfOk, binOk, encodingOk, 2,
                             zFullname, &reason);
@@ -2017,6 +2014,7 @@ static int tagCmp(const void *a, const void *b){
 **                               question.
 **    --no-warnings              omit all warnings about file contents
 **    --nosign                   do not attempt to sign this commit with gpg
+**    --override-lock            allow a check-in even though parent is locked
 **    --private                  do not sync changes and their descendants
 **    --hash                     verify file status using hashing rather
 **                               than relying on file mtimes
@@ -2093,6 +2091,7 @@ void commit_cmd(void){
   allowConflict = find_option("allow-conflict",0,0)!=0;
   allowEmpty = find_option("allow-empty",0,0)!=0;
   allowFork = find_option("allow-fork",0,0)!=0;
+  if( find_option("override-lock",0,0)!=0 ) allowFork = 1;
   allowOlder = find_option("allow-older",0,0)!=0;
   noPrompt = find_option("no-prompt", 0, 0)!=0;
   noWarningFlag = find_option("no-warnings", 0, 0)!=0;
@@ -2128,7 +2127,11 @@ void commit_cmd(void){
 
   /* Do not allow the creation of a new branch using an existing open
   ** branch name unless the --force flag is used */
-  if( sCiInfo.zBranch!=0 && !forceFlag && branch_is_open(sCiInfo.zBranch) ){
+  if( sCiInfo.zBranch!=0
+   && !forceFlag
+   && fossil_strcmp(sCiInfo.zBranch,"private")!=0
+   && branch_is_open(sCiInfo.zBranch)
+  ){
     fossil_fatal("an open branch named \"%s\" already exists - use --force"
                  " to override", sCiInfo.zBranch);
   }
@@ -2162,7 +2165,11 @@ void commit_cmd(void){
   ** Autosync if autosync is enabled and this is not a private check-in.
   */
   if( !g.markPrivate ){
-    if( autosync_loop(SYNC_PULL, db_get_int("autosync-tries", 1), 1) ){
+    int syncFlags = SYNC_PULL;
+    if( vid!=0 && !allowFork && !forceFlag ){
+      syncFlags |= SYNC_CKIN_LOCK;
+    }
+    if( autosync_loop(syncFlags, db_get_int("autosync-tries", 1), 1) ){
       fossil_exit(1);
     }
   }
@@ -2269,18 +2276,20 @@ void commit_cmd(void){
   ** or --force flags is used, or unless this is a private check-in.
   ** The initial commit MUST have tags "trunk" and "sym-trunk".
   */
-  if( !vid ){
-    if( sCiInfo.zBranch==0 ){
-      if( allowFork==0 && forceFlag==0 && g.markPrivate==0
-        && db_exists("SELECT 1 from event where type='ci'") ){
-        fossil_fatal("would fork.  \"update\" first or use --allow-fork.");
-      }
-      sCiInfo.zBranch = db_get("main-branch", "trunk");
-    }
-  }else if( sCiInfo.zBranch==0 && allowFork==0 && forceFlag==0
-    && g.markPrivate==0 && !is_a_leaf(vid)
+  if( sCiInfo.zBranch==0
+   && allowFork==0
+   && forceFlag==0
+   && g.markPrivate==0
+   && (vid==0 || !is_a_leaf(vid) || g.ckinLockFail)
   ){
-    fossil_fatal("would fork.  \"update\" first or use --allow-fork.");
+    if( g.ckinLockFail ){
+      fossil_fatal("Might fork due to a check-in race with user \"%s\"\n"
+                   "Try \"update\" first, or --branch, or use --override-lock",
+                   g.ckinLockFail);
+    }else{
+      fossil_fatal("Would fork.  \"update\" first or use --branch or "
+                   "--allow-fork.");
+    }
   }
 
   /*
@@ -2307,7 +2316,7 @@ void commit_cmd(void){
     blob_append(&comment, zComment, -1);
   }else if( zComFile ){
     blob_zero(&comment);
-    blob_read_from_file(&comment, zComFile);
+    blob_read_from_file(&comment, zComFile, ExtFILE);
     blob_to_utf8_no_bom(&comment, 1);
   }else if( dryRunFlag ){
     blob_zero(&comment);
@@ -2378,12 +2387,7 @@ void commit_cmd(void){
     encodingOk = db_column_int(&q, 5);
 
     blob_zero(&content);
-    if( file_wd_islink(zFullname) ){
-      /* Instead of file content, put link destination path */
-      blob_read_link(&content, zFullname);
-    }else{
-      blob_read_from_file(&content, zFullname);
-    }
+    blob_read_from_file(&content, zFullname, RepoFILE);
     /* Do not emit any warnings when they are disabled. */
     if( !noWarningFlag ){
       abortCommit |= commit_warning(&content, crlfOk, binOk,
@@ -2402,9 +2406,10 @@ void commit_cmd(void){
     nrid = content_put(&content);
     blob_reset(&content);
     if( rid>0 ){
-      content_deltify(rid, nrid, 0);
+      content_deltify(rid, &nrid, 1, 0);
     }
-    db_multi_exec("UPDATE vfile SET mrid=%d, rid=%d WHERE id=%d", nrid,nrid,id);
+    db_multi_exec("UPDATE vfile SET mrid=%d, rid=%d, mhash=NULL WHERE id=%d",
+                  nrid,nrid,id);
     db_multi_exec("INSERT OR IGNORE INTO unsent VALUES(%d)", nrid);
   }
   db_finalize(&q);
@@ -2495,7 +2500,7 @@ void commit_cmd(void){
     zManifestFile = mprintf("%smanifest", g.zLocalRoot);
     blob_write_to_file(&manifest, zManifestFile);
     blob_reset(&manifest);
-    blob_read_from_file(&manifest, zManifestFile);
+    blob_read_from_file(&manifest, zManifestFile, ExtFILE);
     free(zManifestFile);
   }
 
@@ -2509,11 +2514,10 @@ void commit_cmd(void){
     fossil_fatal("%s", g.zErrMsg);
   }
   assert( blob_is_reset(&manifest) );
-  content_deltify(vid, nvid, 0);
+  content_deltify(vid, &nvid, 1, 0);
   zUuid = db_text(0, "SELECT uuid FROM blob WHERE rid=%d", nvid);
 
-  db_prepare(&q, "SELECT uuid,merge FROM vmerge JOIN blob ON merge=rid"
-                 " WHERE id=-4");
+  db_prepare(&q, "SELECT mhash,merge FROM vmerge WHERE id=-4");
   while( db_step(&q)==SQLITE_ROW ){
     const char *zIntegrateUuid = db_column_text(&q, 0);
     if( is_a_leaf(db_column_int(&q, 1)) ){
@@ -2539,11 +2543,11 @@ void commit_cmd(void){
     "DELETE FROM vfile WHERE (vid!=%d OR deleted) AND is_selected(id);"
     "DELETE FROM vmerge;"
     "UPDATE vfile SET vid=%d;"
-    "UPDATE vfile SET rid=mrid, chnged=0, deleted=0, origname=NULL"
+    "UPDATE vfile SET rid=mrid, mhash=NULL, chnged=0, deleted=0, origname=NULL"
     " WHERE is_selected(id);"
     , vid, nvid
   );
-  db_lset_int("checkout", nvid);
+  db_set_checkout(nvid);
 
   /* Update the isexe and islink columns of the vfile table */
   db_prepare(&q,
@@ -2619,7 +2623,9 @@ void commit_cmd(void){
   }
 
   if( !g.markPrivate ){
-    autosync_loop(SYNC_PUSH|SYNC_PULL, db_get_int("autosync-tries", 1), 0);
+    int syncFlags = SYNC_PUSH | SYNC_PULL | SYNC_IFABLE;
+    int nTries = db_get_int("autosync-tries",1);
+    autosync_loop(syncFlags, nTries, 0);
   }
   if( count_nonbranch_children(vid)>1 ){
     fossil_print("**** warning: a fork has occurred *****\n");
