@@ -37,11 +37,11 @@
 #define CONFIGSET_USER      0x000020     /* The USER table */
 #define CONFIGSET_ADDR      0x000040     /* The CONCEALED table */
 #define CONFIGSET_XFER      0x000080     /* Transfer configuration */
-
-#define CONFIGSET_ALL       0x0000ff     /* Everything */
+#define CONFIGSET_ALIAS     0x000100     /* URL Aliases */
+#define CONFIGSET_SCRIBER   0x000200     /* Email subscribers */
+#define CONFIGSET_ALL       0x0003ff     /* Everything */
 
 #define CONFIGSET_OVERWRITE 0x100000     /* Causes overwrite instead of merge */
-#define CONFIGSET_OLDFORMAT 0x200000     /* Use the legacy format */
 
 /*
 ** This mask is used for the common TH1 configuration settings (i.e. those
@@ -60,16 +60,18 @@ static struct {
   int groupMask;       /* Mask for that configuration set */
   const char *zHelp;   /* What it does */
 } aGroupName[] = {
-  { "/email",        CONFIGSET_ADDR,  "Concealed email addresses in tickets" },
-  { "/project",      CONFIGSET_PROJ,  "Project name and description"         },
-  { "/skin",         CONFIGSET_SKIN | CONFIGSET_CSS,
-                                      "Web interface appearance settings"    },
-  { "/css",          CONFIGSET_CSS,   "Style sheet"                          },
-  { "/shun",         CONFIGSET_SHUN,  "List of shunned artifacts"            },
-  { "/ticket",       CONFIGSET_TKT,   "Ticket setup",                        },
-  { "/user",         CONFIGSET_USER,  "Users and privilege settings"         },
-  { "/xfer",         CONFIGSET_XFER,  "Transfer setup",                      },
-  { "/all",          CONFIGSET_ALL,   "All of the above"                     },
+  { "/email",       CONFIGSET_ADDR,  "Concealed email addresses in tickets" },
+  { "/project",     CONFIGSET_PROJ,  "Project name and description"         },
+  { "/skin",        CONFIGSET_SKIN | CONFIGSET_CSS,
+                                     "Web interface appearance settings"    },
+  { "/css",         CONFIGSET_CSS,   "Style sheet"                          },
+  { "/shun",        CONFIGSET_SHUN,  "List of shunned artifacts"            },
+  { "/ticket",      CONFIGSET_TKT,   "Ticket setup",                        },
+  { "/user",        CONFIGSET_USER,  "Users and privilege settings"         },
+  { "/xfer",        CONFIGSET_XFER,  "Transfer setup",                      },
+  { "/alias",       CONFIGSET_ALIAS, "URL Aliases",                         },
+  { "/subscriber",  CONFIGSET_SCRIBER,"Email notification subscriber list"  },
+  { "/all",         CONFIGSET_ALL,   "All of the above"                     },
 };
 
 
@@ -94,11 +96,20 @@ static struct {
   { "background-mimetype",    CONFIGSET_SKIN },
   { "background-image",       CONFIGSET_SKIN },
   { "timeline-block-markup",  CONFIGSET_SKIN },
+  { "timeline-date-format",   CONFIGSET_SKIN },
+  { "timeline-dwelltime",     CONFIGSET_SKIN },
+  { "timeline-closetime",     CONFIGSET_SKIN },
   { "timeline-max-comment",   CONFIGSET_SKIN },
   { "timeline-plaintext",     CONFIGSET_SKIN },
+  { "timeline-truncate-at-blank", CONFIGSET_SKIN },
+  { "timeline-utc",           CONFIGSET_SKIN },
   { "adunit",                 CONFIGSET_SKIN },
   { "adunit-omit-if-admin",   CONFIGSET_SKIN },
   { "adunit-omit-if-user",    CONFIGSET_SKIN },
+  { "sitemap-docidx",         CONFIGSET_SKIN },
+  { "sitemap-download",       CONFIGSET_SKIN },
+  { "sitemap-license",        CONFIGSET_SKIN },
+  { "sitemap-contact",        CONFIGSET_SKIN },
 
 #ifdef FOSSIL_ENABLE_TH1_DOCS
   { "th1-docs",               CONFIGSET_TH1 },
@@ -132,6 +143,7 @@ static struct {
   { "parent-project-code",    CONFIGSET_PROJ },
   { "parent-project-name",    CONFIGSET_PROJ },
   { "hash-policy",            CONFIGSET_PROJ },
+  { "comment-format",         CONFIGSET_PROJ },
 
 #ifdef FOSSIL_ENABLE_LEGACY_MV_RM
   { "mv-rm-files",            CONFIGSET_PROJ },
@@ -156,6 +168,10 @@ static struct {
 
   { "@shun",                  CONFIGSET_SHUN },
 
+  { "@alias",                 CONFIGSET_ALIAS },
+
+  { "@subscriber",            CONFIGSET_SCRIBER },
+
   { "xfer-common-script",     CONFIGSET_XFER },
   { "xfer-push-script",       CONFIGSET_XFER },
   { "xfer-commit-script",     CONFIGSET_XFER },
@@ -172,25 +188,15 @@ const char *configure_first_name(int iMask){
   return configure_next_name(iMask);
 }
 const char *configure_next_name(int iMask){
-  if( iMask & CONFIGSET_OLDFORMAT ){
-    while( iConfig<count(aConfig) ){
-      if( aConfig[iConfig].groupMask & iMask ){
-        return aConfig[iConfig++].zName;
-      }else{
-        iConfig++;
-      }
-    }
-  }else{
-    if( iConfig==0 && (iMask & CONFIGSET_ALL)==CONFIGSET_ALL ){
-      iConfig = count(aGroupName);
-      return "/all";
-    }
-    while( iConfig<count(aGroupName)-1 ){
-      if( aGroupName[iConfig].groupMask & iMask ){
-        return aGroupName[iConfig++].zName;
-      }else{
-        iConfig++;
-      }
+  if( iConfig==0 && (iMask & CONFIGSET_ALL)==CONFIGSET_ALL ){
+    iConfig = count(aGroupName);
+    return "/all";
+  }
+  while( iConfig<count(aGroupName)-1 ){
+    if( aGroupName[iConfig].groupMask & iMask ){
+      return aGroupName[iConfig++].zName;
+    }else{
+      iConfig++;
     }
   }
   return 0;
@@ -222,7 +228,7 @@ const char *configure_inop_rhs(int iMask){
 ** Return the mask for the named configuration parameter if it can be
 ** safely exported.  Return 0 if the parameter is not safe to export.
 **
-** "Safe" in the previous paragraph means the permission is created to
+** "Safe" in the previous paragraph means the permission is granted to
 ** export the property.  In other words, the requesting side has presented
 ** login credentials and has sufficient capabilities to access the requested
 ** information.
@@ -238,7 +244,7 @@ int configure_is_exportable(const char *zName){
     if( strncmp(zName, aConfig[i].zName, n)==0 && aConfig[i].zName[n]==0 ){
       int m = aConfig[i].groupMask;
       if( !g.perm.Admin ){
-        m &= ~CONFIGSET_USER;
+        m &= ~(CONFIGSET_USER|CONFIGSET_SCRIBER);
       }
       if( !g.perm.RdAddr ){
         m &= ~CONFIGSET_ADDR;
@@ -246,185 +252,16 @@ int configure_is_exportable(const char *zName){
       return m;
     }
   }
+  if( strncmp(zName, "walias:/", 8)==0 ){
+    return CONFIGSET_ALIAS;
+  }
   return 0;
 }
 
 /*
-** zName is one of the special configuration names that refers to an entire
-** table rather than a single entry in CONFIG.  Special names are "@reportfmt"
-** and "@shun" and "@user".  This routine writes SQL text into pOut that when
-** evaluated will populate the corresponding table with data.
-*/
-void configure_render_special_name(const char *zName, Blob *pOut){
-  Stmt q;
-  if( fossil_strcmp(zName, "@shun")==0 ){
-    db_prepare(&q, "SELECT uuid FROM shun");
-    while( db_step(&q)==SQLITE_ROW ){
-      blob_appendf(pOut, "INSERT OR IGNORE INTO shun VALUES('%s');\n",
-        db_column_text(&q, 0)
-      );
-    }
-    db_finalize(&q);
-  }else if( fossil_strcmp(zName, "@reportfmt")==0 ){
-    db_prepare(&q, "SELECT title, cols, sqlcode FROM reportfmt");
-    while( db_step(&q)==SQLITE_ROW ){
-      blob_appendf(pOut, "INSERT INTO _xfer_reportfmt(title,cols,sqlcode)"
-                         " VALUES(%Q,%Q,%Q);\n",
-        db_column_text(&q, 0),
-        db_column_text(&q, 1),
-        db_column_text(&q, 2)
-      );
-    }
-    db_finalize(&q);
-  }else if( fossil_strcmp(zName, "@user")==0 ){
-    db_prepare(&q,
-        "SELECT login, CASE WHEN length(pw)==40 THEN pw END,"
-        "       cap, info, quote(photo) FROM user");
-    while( db_step(&q)==SQLITE_ROW ){
-      blob_appendf(pOut, "INSERT INTO _xfer_user(login,pw,cap,info,photo)"
-                         " VALUES(%Q,%Q,%Q,%Q,%s);\n",
-        db_column_text(&q, 0),
-        db_column_text(&q, 1),
-        db_column_text(&q, 2),
-        db_column_text(&q, 3),
-        db_column_text(&q, 4)
-      );
-    }
-    db_finalize(&q);
-  }else if( fossil_strcmp(zName, "@concealed")==0 ){
-    db_prepare(&q, "SELECT hash, content FROM concealed");
-    while( db_step(&q)==SQLITE_ROW ){
-      blob_appendf(pOut, "INSERT OR IGNORE INTO concealed(hash,content)"
-                         " VALUES(%Q,%Q);\n",
-        db_column_text(&q, 0),
-        db_column_text(&q, 1)
-      );
-    }
-    db_finalize(&q);
-  }
-}
-
-/*
-** Two SQL functions:
-**
-**        config_is_reset(int)
-**        config_reset(int)
-**
-** The config_is_reset() function takes the integer valued argument and
-** ANDs it against the static variable "configHasBeenReset" below.  The
-** function returns TRUE or FALSE depending on the result depending on
-** whether or not the corresponding configuration table has been reset.  The
-** config_reset() function adds the bits to "configHasBeenReset" that
-** are given in the argument.
-**
-** These functions are used below in the WHEN clause of a trigger to
-** get the trigger to fire exactly once.
+** A mask of all configuration tables that have been reset already.
 */
 static int configHasBeenReset = 0;
-static void config_is_reset_function(
-  sqlite3_context *context,
-  int argc,
-  sqlite3_value **argv
-){
-  int m = sqlite3_value_int(argv[0]);
-  sqlite3_result_int(context, (configHasBeenReset&m)!=0 );
-}
-static void config_reset_function(
-  sqlite3_context *context,
-  int argc,
-  sqlite3_value **argv
-){
-  int m = sqlite3_value_int(argv[0]);
-  configHasBeenReset |= m;
-}
-
-/*
-** Create the temporary _xfer_reportfmt and _xfer_user tables that are
-** necessary in order to evaluate the SQL text generated by the
-** configure_render_special_name() routine.
-**
-** If replaceFlag is true, then the setup is such that the content in
-** the SQL text will completely replace the current repository configuration.
-** If replaceFlag is false, then the SQL text will be merged with the
-** existing configuration.  When merging, existing values take priority
-** over SQL text values.
-*/
-void configure_prepare_to_receive(int replaceFlag){
-  static const char zSQL1[] =
-    @ CREATE TEMP TABLE _xfer_reportfmt(
-    @    rn integer primary key,  -- Report number
-    @    owner text,              -- Owner of this report format (not used)
-    @    title text UNIQUE ON CONFLICT IGNORE,  -- Title of this report
-    @    cols text,               -- A color-key specification
-    @    sqlcode text             -- An SQL SELECT statement for this report
-    @ );
-    @ CREATE TEMP TABLE _xfer_user(
-    @   uid INTEGER PRIMARY KEY,        -- User ID
-    @   login TEXT UNIQUE ON CONFLICT IGNORE,   -- login name of the user
-    @   pw TEXT,                        -- password
-    @   cap TEXT,                       -- Capabilities of this user
-    @   cookie TEXT,                    -- WWW login cookie
-    @   ipaddr TEXT,                    -- IP address for which cookie is valid
-    @   cexpire DATETIME,               -- Time when cookie expires
-    @   info TEXT,                      -- contact information
-    @   photo BLOB                      -- JPEG image of this user
-    @ );
-    @ INSERT INTO _xfer_reportfmt
-    @    SELECT rn,owner,title,cols,sqlcode FROM reportfmt;
-    @ INSERT INTO _xfer_user
-    @    SELECT uid,login,pw,cap,cookie,ipaddr,cexpire,info,photo FROM user;
-  ;
-  assert( strchr(zSQL1,'%')==0 );
-  db_multi_exec(zSQL1 /*works-like:""*/);
-
-  /* When the replace flag is set, add triggers that run the first time
-  ** that new data is seen.  The triggers run only once and delete all the
-  ** existing data.
-  */
-  if( replaceFlag ){
-    static const char zSQL2[] =
-      @ CREATE TRIGGER _xfer_r1 BEFORE INSERT ON _xfer_reportfmt
-      @ WHEN NOT config_is_reset(2) BEGIN
-      @   DELETE FROM _xfer_reportfmt;
-      @   SELECT config_reset(2);
-      @ END;
-      @ CREATE TRIGGER _xfer_r2 BEFORE INSERT ON _xfer_user
-      @ WHEN NOT config_is_reset(16) BEGIN
-      @   DELETE FROM _xfer_user;
-      @   SELECT config_reset(16);
-      @ END;
-      @ CREATE TEMP TRIGGER _xfer_r3 BEFORE INSERT ON shun
-      @ WHEN NOT config_is_reset(8) BEGIN
-      @   DELETE FROM shun;
-      @   SELECT config_reset(8);
-      @ END;
-    ;
-    sqlite3_create_function(g.db, "config_is_reset", 1, SQLITE_UTF8, 0,
-         config_is_reset_function, 0, 0);
-    sqlite3_create_function(g.db, "config_reset", 1, SQLITE_UTF8, 0,
-         config_reset_function, 0, 0);
-    configHasBeenReset = 0;
-    assert( strchr(zSQL2,'%')==0 );
-    db_multi_exec(zSQL2 /*works-like:""*/);
-  }
-}
-
-/*
-** After receiving configuration data, call this routine to transfer
-** the results into the main database.
-*/
-void configure_finalize_receive(void){
-  static const char zSQL[] =
-    @ DELETE FROM user;
-    @ INSERT INTO user SELECT * FROM _xfer_user;
-    @ DELETE FROM reportfmt;
-    @ INSERT INTO reportfmt SELECT * FROM _xfer_reportfmt;
-    @ DROP TABLE _xfer_user;
-    @ DROP TABLE _xfer_reportfmt;
-  ;
-  assert( strchr(zSQL,'%')==0 );
-  db_multi_exec(zSQL /*works-like:""*/);
-}
 
 /*
 ** Mask of modified configuration sets
@@ -490,7 +327,11 @@ static int safeInt(const char *z){
 **
 ** NEW FORMAT:
 **
-** zName is one of "/config", "/user", "/shun", "/reportfmt", or "/concealed".
+** zName is one of:
+**
+**     "/config", "/user",  "/shun", "/reportfmt", "/concealed",
+**     "/subscriber",
+**
 ** zName indicates the table that holds the configuration information being
 ** transferred.  pContent is a string that consist of alternating Fossil
 ** and SQL tokens.  The First token is a timestamp in seconds since 1970.
@@ -510,48 +351,40 @@ static int safeInt(const char *z){
 **    /shun       $MTIME $UUID scom $VALUE
 **    /reportfmt  $MTIME $TITLE owner $VALUE cols $VALUE sqlcode $VALUE
 **    /concealed  $MTIME $HASH content $VALUE
-**
-** OLD FORMAT:
-**
-** The old format is retained for backwards compatibility, but is deprecated.
-** The cutover from old format to new was on 2011-04-25.  After sufficient
-** time has passed, support for the old format will be removed.
-**
-** zName is either the NAME of an element of the CONFIG table, or else
-** one of the special names "@shun", "@reportfmt", "@user", or "@concealed".
-** If zName is a CONFIG table name, then CONTENT replaces (overwrites) the
-** element in the CONFIG table.  For one of the @-labels, CONTENT is raw
-** SQL that is evaluated.  Note that the raw SQL in CONTENT might not
-** insert directly into the target table but might instead use a proxy
-** table like _fer_reportfmt or _xfer_user.  Such tables must be created
-** ahead of time using configure_prepare_to_receive().  Then after multiple
-** calls to this routine, configure_finalize_receive() to transfer the
-** information received into the true target table.
+**    /subscriber $SMTIME $SEMAIL suname $V ...
 */
 void configure_receive(const char *zName, Blob *pContent, int groupMask){
+  int checkMask;   /* Masks for which we must first check existance of tables */
+
+  checkMask = CONFIGSET_SCRIBER;
   if( zName[0]=='/' ){
     /* The new format */
-    char *azToken[12];
+    char *azToken[24];
     int nToken = 0;
     int ii, jj;
     int thisMask;
     Blob name, value, sql;
     static const struct receiveType {
-      const char *zName;
-      const char *zPrimKey;
-      int nField;
-      const char *azField[4];
+      const char *zName;         /* Configuration key for this table */
+      const char *zPrimKey;      /* Primary key column */
+      int nField;                /* Number of data fields */
+      const char *azField[6];    /* Names of the data fields */
     } aType[] = {
-      { "/config",    "name",  1, { "value", 0, 0, 0 }              },
-      { "@user",      "login", 4, { "pw", "cap", "info", "photo" }  },
-      { "@shun",      "uuid",  1, { "scom", 0, 0, 0 }               },
-      { "@reportfmt", "title", 3, { "owner", "cols", "sqlcode", 0 } },
-      { "@concealed", "hash",  1, { "content", 0, 0, 0 }            },
+      { "/config",    "name",  1, { "value", 0,0,0,0,0 }           },
+      { "@user",      "login", 4, { "pw","cap","info","photo",0,0} },
+      { "@shun",      "uuid",  1, { "scom", 0,0,0,0,0}             },
+      { "@reportfmt", "title", 3, { "owner","cols","sqlcode",0,0,0}},
+      { "@concealed", "hash",  1, { "content", 0,0,0,0,0 }         },
+      { "@subscriber","semail",6,
+         { "suname","sdigest","sdonotcall","ssub","sctime","smip"}         },
     };
+
+    /* Locate the receiveType in aType[ii] */
     for(ii=0; ii<count(aType); ii++){
       if( fossil_strcmp(&aType[ii].zName[1],&zName[1])==0 ) break;
     }
     if( ii>=count(aType) ) return;
+
     while( blob_token(pContent, &name) && blob_sqltoken(pContent, &value) ){
       char *z = blob_terminate(&name);
       if( !safeSql(z) ) return;
@@ -566,7 +399,7 @@ void configure_receive(const char *zName, Blob *pContent, int groupMask){
       azToken[nToken++] = z;
       azToken[nToken++] = z = blob_terminate(&value);
       if( !safeSql(z) ) return;
-      if( nToken>=count(azToken) ) break;
+      if( nToken>=count(azToken)-1 ) break;
     }
     if( nToken<2 ) return;
     if( aType[ii].zName[0]=='/' ){
@@ -575,6 +408,12 @@ void configure_receive(const char *zName, Blob *pContent, int groupMask){
       thisMask = configure_is_exportable(aType[ii].zName);
     }
     if( (thisMask & groupMask)==0 ) return;
+    if( (thisMask & checkMask)!=0 ){
+      if( (thisMask & CONFIGSET_SCRIBER)!=0 ){
+        alert_schema(1);
+      }
+      checkMask &= ~thisMask;
+    }
 
     blob_zero(&sql);
     if( groupMask & CONFIGSET_OVERWRITE ){
@@ -586,12 +425,14 @@ void configure_receive(const char *zName, Blob *pContent, int groupMask){
     }else{
       blob_append_sql(&sql, "INSERT OR IGNORE INTO ");
     }
-    blob_append_sql(&sql, "\"%w\"(\"%w\", mtime", &zName[1], aType[ii].zPrimKey);
+    blob_append_sql(&sql, "\"%w\"(\"%w\",mtime",
+         &zName[1], aType[ii].zPrimKey);
+    if( fossil_stricmp(zName,"/subscriber")==0 ) alert_schema(0);
     for(jj=2; jj<nToken; jj+=2){
        blob_append_sql(&sql, ",\"%w\"", azToken[jj]);
     }
     blob_append_sql(&sql,") VALUES(%s,%s",
-       azToken[1] /*safe-for-%s*/, azToken[0] /*safe-for-%s*/);
+       azToken[1] /*safe-for-%s*/, azToken[0]/*safe-for-%s*/);
     for(jj=2; jj<nToken; jj+=2){
        blob_append_sql(&sql, ",%s", azToken[jj+1] /*safe-for-%s*/);
     }
@@ -611,31 +452,6 @@ void configure_receive(const char *zName, Blob *pContent, int groupMask){
     }
     blob_reset(&sql);
     rebuildMask |= thisMask;
-  }else{
-    /* Otherwise, the old format */
-    if( (configure_is_exportable(zName) & groupMask)==0 ) return;
-    if( fossil_strcmp(zName, "logo-image")==0 ){
-      Stmt ins;
-      db_prepare(&ins,
-        "REPLACE INTO config(name, value, mtime) VALUES(:name, :value, now())"
-      );
-      db_bind_text(&ins, ":name", zName);
-      db_bind_blob(&ins, ":value", pContent);
-      db_step(&ins);
-      db_finalize(&ins);
-    }else if( zName[0]=='@' ){
-      /* Notice that we are evaluating arbitrary SQL received from the
-      ** client.  But this can only happen if the client has authenticated
-      ** as an administrator, so presumably we trust the client at this
-      ** point.
-      */
-      db_multi_exec("%s", blob_str(pContent) /*safe-for-%s*/);
-    }else{
-      db_multi_exec(
-         "REPLACE INTO config(name,value,mtime) VALUES(%Q,%Q,now())",
-         zName, blob_str(pContent)
-      );
-    }
   }
 }
 
@@ -760,6 +576,51 @@ int configure_send_group(
     }
     db_finalize(&q);
   }
+  if( groupMask & CONFIGSET_ALIAS ){
+    db_prepare(&q, "SELECT mtime, quote(name), quote(value) FROM config"
+                   " WHERE name GLOB 'walias:/*' AND mtime>=%lld", iStart);
+    while( db_step(&q)==SQLITE_ROW ){
+      blob_appendf(&rec,"%s %s value %s",
+        db_column_text(&q, 0),
+        db_column_text(&q, 1),
+        db_column_text(&q, 2)
+      );
+      blob_appendf(pOut, "config /config %d\n%s\n",
+                   blob_size(&rec), blob_str(&rec));
+      nCard++;
+      blob_reset(&rec);
+    }
+    db_finalize(&q);
+  }
+  if( (groupMask & CONFIGSET_SCRIBER)!=0
+   && db_table_exists("repository","subscriber")
+  ){
+    db_prepare(&q, "SELECT mtime, quote(semail),"
+                   " quote(suname), quote(sdigest),"
+                   " quote(sdonotcall), quote(ssub),"
+                   " quote(sctime), quote(smip)"
+                   " FROM subscriber WHERE sverified"
+                   " AND mtime>=%lld", iStart);
+    while( db_step(&q)==SQLITE_ROW ){
+      blob_appendf(&rec,
+        "%lld %s suname %s sdigest %s sdonotcall %s ssub %s"
+        " sctime %s smip %s",
+        db_column_int64(&q, 0), /* mtime */
+        db_column_text(&q, 1),  /* semail (PK) */
+        db_column_text(&q, 2),  /* suname */
+        db_column_text(&q, 3),  /* sdigest */
+        db_column_text(&q, 4),  /* sdonotcall */
+        db_column_text(&q, 5),  /* ssub */
+        db_column_text(&q, 6),  /* sctime */
+        db_column_text(&q, 7)   /* smip */
+      );
+      blob_appendf(pOut, "config /subscriber %d\n%s\n",
+                   blob_size(&rec), blob_str(&rec));
+      nCard++;
+      blob_reset(&rec);
+    }
+    db_finalize(&q);
+  }
   db_prepare(&q, "SELECT mtime, quote(name), quote(value) FROM config"
                  " WHERE name=:name AND mtime>=%lld", iStart);
   for(ii=0; ii<count(aConfig); ii++){
@@ -798,7 +659,8 @@ int configure_name_to_mask(const char *z, int notFoundIsFatal){
   if( notFoundIsFatal ){
     fossil_print("Available configuration areas:\n");
     for(i=0; i<count(aGroupName); i++){
-      fossil_print("  %-10s %s\n", &aGroupName[i].zName[1], aGroupName[i].zHelp);
+      fossil_print("  %-13s %s\n",
+            &aGroupName[i].zName[1], aGroupName[i].zHelp);
     }
     fossil_fatal("no such configuration area: \"%s\"", z);
   }
@@ -841,7 +703,9 @@ static void export_config(
 **    %fossil configuration export AREA FILENAME
 **
 **         Write to FILENAME exported configuration information for AREA.
-**         AREA can be one of:  all email project shun skin ticket user
+**         AREA can be one of:
+**
+**             all email project shun skin ticket user alias subscriber
 **
 **    %fossil configuration import FILENAME
 **
@@ -858,10 +722,8 @@ static void export_config(
 **
 **         Pull and install the configuration from a different server
 **         identified by URL.  If no URL is specified, then the default
-**         server is used. Use the --legacy option for the older protocol
-**         (when talking to servers compiled prior to 2011-04-27.)  Use
-**         the --overwrite flag to completely replace local settings with
-**         content received from URL.
+**         server is used.  Use the --overwrite flag to completely
+**         replace local settings with content received from URL.
 **
 **    %fossil configuration push AREA ?URL?
 **
@@ -869,7 +731,6 @@ static void export_config(
 **         by URL.  Admin privilege is required on the remote server for
 **         this to work.  When the same record exists both locally and on
 **         the remote end, the one that was most recently changed wins.
-**         Use the --legacy flag when talking to older servers.
 **
 **    %fossil configuration reset AREA
 **
@@ -918,7 +779,7 @@ void configuration_cmd(void){
     Blob in;
     int groupMask;
     if( g.argc!=4 ) usage(mprintf("%s FILENAME",zMethod));
-    blob_read_from_file(&in, g.argv[3]);
+    blob_read_from_file(&in, g.argv[3], ExtFILE);
     db_begin_transaction();
     if( zMethod[0]=='i' ){
       groupMask = CONFIGSET_ALL | CONFIGSET_OVERWRITE;
@@ -934,10 +795,8 @@ void configuration_cmd(void){
   ){
     int mask;
     const char *zServer = 0;
-    int legacyFlag = 0;
     int overwriteFlag = 0;
 
-    if( zMethod[0]!='s' ) legacyFlag = find_option("legacy",0,0)!=0;
     if( strncmp(zMethod,"pull",n)==0 ){
       overwriteFlag = find_option("overwrite",0,0)!=0;
     }
@@ -953,7 +812,6 @@ void configuration_cmd(void){
     if( g.url.protocol==0 ) fossil_fatal("no server URL specified");
     user_select();
     url_enable_proxy("via proxy: ");
-    if( legacyFlag ) mask |= CONFIGSET_OLDFORMAT;
     if( overwriteFlag ) mask |= CONFIGSET_OVERWRITE;
     if( strncmp(zMethod, "push", n)==0 ){
       client_sync(0,0,(unsigned)mask);
@@ -984,6 +842,15 @@ void configuration_cmd(void){
         db_multi_exec("DELETE FROM concealed");
       }else if( fossil_strcmp(zName,"@shun")==0 ){
         db_multi_exec("DELETE FROM shun");
+      }else if( fossil_strcmp(zName,"@subscriber")==0 ){
+        if( db_table_exists("repository","subscriber") ){
+          db_multi_exec("DELETE FROM subscriber");
+        }
+      }else if( fossil_strcmp(zName,"@forum")==0 ){
+        if( db_table_exists("repository","forumpost") ){
+          db_multi_exec("DELETE FROM forumpost");
+          db_multi_exec("DELETE FROM forumthread");
+        }
       }else if( fossil_strcmp(zName,"@reportfmt")==0 ){
         db_multi_exec("DELETE FROM reportfmt");
         assert( strchr(zRepositorySchemaDefaultReports,'%')==0 );
@@ -1175,9 +1042,9 @@ void test_var_set_cmd(void){
   zVar = g.argv[2];
   if( zFile ){
     if( zBlob ) fossil_fatal("cannot do both --file or --blob");
-    blob_read_from_file(&x, zFile);
+    blob_read_from_file(&x, zFile, ExtFILE);
   }else if( zBlob ){
-    blob_read_from_file(&x, zBlob);
+    blob_read_from_file(&x, zBlob, ExtFILE);
   }else{
     blob_init(&x,g.argv[3],-1);
   }
