@@ -182,14 +182,10 @@ void diff_file(
 
     /* Read content of zFile2 into memory */
     blob_zero(&file2);
-    if( file_wd_size(zFile2)<0 ){
+    if( file_size(zFile2, ExtFILE)<0 ){
       zName2 = NULL_DEVICE;
     }else{
-      if( file_wd_islink(0) ){
-        blob_read_link(&file2, zFile2);
-      }else{
-        blob_read_from_file(&file2, zFile2);
-      }
+      blob_read_from_file(&file2, zFile2, ExtFILE);
       zName2 = zName;
     }
 
@@ -219,7 +215,6 @@ void diff_file(
     /* Release memory resources */
     blob_reset(&file2);
   }else{
-    int cnt = 0;
     Blob nameFile1;    /* Name of temporary file to old pFile1 content */
     Blob cmd;          /* Text of command to run */
 
@@ -239,12 +234,8 @@ void diff_file(
         glob_free(pBinary);
       }
       blob_zero(&file2);
-      if( file_wd_size(zFile2)>=0 ){
-        if( file_wd_islink(0) ){
-          blob_read_link(&file2, zFile2);
-        }else{
-          blob_read_from_file(&file2, zFile2);
-        }
+      if( file_size(zFile2, ExtFILE)>=0 ){
+        blob_read_from_file(&file2, zFile2, ExtFILE);
       }
       if( looks_like_binary(&file2) ){
         fossil_print("%s",DIFF_CANNOT_COMPUTE_BINARY);
@@ -256,24 +247,18 @@ void diff_file(
 
     /* Construct a temporary file to hold pFile1 based on the name of
     ** zFile2 */
-    blob_zero(&nameFile1);
-    do{
-      blob_reset(&nameFile1);
-      blob_appendf(&nameFile1, "%s~%d", zFile2, cnt++);
-    }while( file_access(blob_str(&nameFile1),F_OK)==0 );
+    file_tempname(&nameFile1, zFile2, "orig");
     blob_write_to_file(pFile1, blob_str(&nameFile1));
 
     /* Construct the external diff command */
     blob_zero(&cmd);
-    blob_appendf(&cmd, "%s ", zDiffCmd);
+    blob_append(&cmd, zDiffCmd, -1);
     if( fSwapDiff ){
-      shell_escape(&cmd, zFile2);
-      blob_append(&cmd, " ", 1);
-      shell_escape(&cmd, blob_str(&nameFile1));
+      blob_append_escaped_arg(&cmd, zFile2);
+      blob_append_escaped_arg(&cmd, blob_str(&nameFile1));
     }else{
-      shell_escape(&cmd, blob_str(&nameFile1));
-      blob_append(&cmd, " ", 1);
-      shell_escape(&cmd, zFile2);
+      blob_append_escaped_arg(&cmd, blob_str(&nameFile1));
+      blob_append_escaped_arg(&cmd, zFile2);
     }
 
     /* Run the external diff command */
@@ -329,8 +314,6 @@ void diff_file_mem(
     Blob cmd;
     Blob temp1;
     Blob temp2;
-    Blob prefix1;
-    Blob prefix2;
 
     if( !fIncludeBinary ){
       if( isBin1 || isBin2 ){
@@ -348,24 +331,17 @@ void diff_file_mem(
       }
     }
 
-    /* Construct a prefix for the temporary file names */
-    blob_zero(&prefix1);
-    blob_zero(&prefix2);
-    blob_appendf(&prefix1, "%s-v1", zName);
-    blob_appendf(&prefix2, "%s-v2", zName);
-
     /* Construct a temporary file names */
-    file_tempname(&temp1, blob_str(&prefix1));
-    file_tempname(&temp2, blob_str(&prefix2));
+    file_tempname(&temp1, zName, "before");
+    file_tempname(&temp2, zName, "after");
     blob_write_to_file(pFile1, blob_str(&temp1));
     blob_write_to_file(pFile2, blob_str(&temp2));
 
     /* Construct the external diff command */
     blob_zero(&cmd);
-    blob_appendf(&cmd, "%s ", zDiffCmd);
-    shell_escape(&cmd, blob_str(&temp1));
-    blob_append(&cmd, " ", 1);
-    shell_escape(&cmd, blob_str(&temp2));
+    blob_append(&cmd, zDiffCmd, -1);
+    blob_append_escaped_arg(&cmd, blob_str(&temp1));
+    blob_append_escaped_arg(&cmd, blob_str(&temp2));
 
     /* Run the external diff command */
     fossil_system(blob_str(&cmd));
@@ -374,8 +350,6 @@ void diff_file_mem(
     file_delete(blob_str(&temp1));
     file_delete(blob_str(&temp2));
 
-    blob_reset(&prefix1);
-    blob_reset(&prefix2);
     blob_reset(&temp1);
     blob_reset(&temp2);
     blob_reset(&cmd);
@@ -494,7 +468,7 @@ static void diff_against_disk(
     if( showDiff ){
       Blob content;
       int isBin;
-      if( !isLink != !file_wd_islink(zFullName) ){
+      if( !isLink != !file_islink(zFullName) ){
         diff_print_index(zPathname, diffFlags);
         diff_print_filenames(zPathname, zPathname, diffFlags);
         fossil_print("%s",DIFF_CANNOT_COMPUTE_SYMLINK);
@@ -725,6 +699,7 @@ void diff_tk(const char *zSubCmd, int firstArg){
   Blob script;
   const char *zTempFile = 0;
   char *zCmd;
+  const char *zTclsh;
   blob_zero(&script);
   blob_appendf(&script, "set fossilcmd {| \"%/\" %s --html -y -i -v",
                g.nameOfExe, zSubCmd);
@@ -732,6 +707,10 @@ void diff_tk(const char *zSubCmd, int firstArg){
   find_option("side-by-side","y",0);
   find_option("internal","i",0);
   find_option("verbose","v",0);
+  zTclsh = find_option("tclsh",0,1);
+  if( zTclsh==0 ){
+    zTclsh = db_get("tclsh","tclsh");
+  }
   /* The undocumented --script FILENAME option causes the Tk script to
   ** be written into the FILENAME instead of being run.  This is used
   ** for testing and debugging. */
@@ -749,7 +728,7 @@ void diff_tk(const char *zSubCmd, int firstArg){
   blob_appendf(&script, "}\n%s", builtin_file("diff.tcl", 0));
   if( zTempFile ){
     blob_write_to_file(&script, zTempFile);
-    fossil_print("To see diff, run: tclsh \"%s\"\n", zTempFile);
+    fossil_print("To see diff, run: %s \"%s\"\n", zTclsh, zTempFile);
   }else{
 #if defined(FOSSIL_ENABLE_TCL)
     Th_FossilInit(TH_INIT_DEFAULT);
@@ -766,7 +745,7 @@ void diff_tk(const char *zSubCmd, int firstArg){
      */
 #endif
     zTempFile = write_blob_to_temp_file(&script);
-    zCmd = mprintf("tclsh \"%s\"", zTempFile);
+    zCmd = mprintf("\"%s\" \"%s\"", zTclsh, zTempFile);
     fossil_system(zCmd);
     file_delete(zTempFile);
     fossil_free(zCmd);
@@ -849,9 +828,11 @@ const char *diff_get_binary_glob(void){
 **   --exec-rel-paths           Force relative path names with external commands.
 **   --from|-r VERSION          Select VERSION as source for the diff
 **   --internal|-i              Use internal diff logic
+**   --new-file|-N              Show complete text of added and deleted files
 **   --numstat                  Show only the number of lines delete and added
 **   --side-by-side|-y          Side-by-side diff
 **   --strip-trailing-cr        Strip trailing CR
+**   --tclsh PATH               Tcl/Tk used for --tk (default: "tclsh")
 **   --tk                       Launch a Tcl/Tk GUI for display
 **   --to VERSION               Select VERSION as target for the diff
 **   --undo                     Diff against the "undo" buffer
@@ -876,7 +857,7 @@ void diff_cmd(void){
   u64 diffFlags = 0;         /* Flags to control the DIFF */
   FileDirList *pFileDir = 0; /* Restrict the diff to these files */
 
-  if( find_option("tk",0,0)!=0 ){
+  if( find_option("tk",0,0)!=0 || has_option("tclsh") ){
     diff_tk("diff", 2);
     return;
   }
@@ -970,7 +951,7 @@ void diff_cmd(void){
     for(i=0; pFileDir[i].zName; i++){
       if( pFileDir[i].nUsed==0
        && strcmp(pFileDir[0].zName,".")!=0
-       && !file_wd_isdir(g.argv[i+2])
+       && !file_isdir(g.argv[i+2], ExtFILE)
       ){
         fossil_fatal("not found: '%s'", g.argv[i+2]);
       }
