@@ -11,6 +11,12 @@
 #
 #      tclsh makemake.tcl
 #
+# Add new source files by listing the files (without their .c suffix)
+# in the "src" variable.  Add new resource files to the "extra_files"
+# variable.  There are other variables that you can alter, down to
+# the "STOP HERE" comment.  The stuff below "STOP HERE" should rarely need
+# to change.
+#
 #############################################################################
 
 # Basenames of all source files that get preprocessed using
@@ -22,8 +28,10 @@
 #
 set src {
   add
+  alerts
   allrepo
   attach
+  backoffice
   bag
   bisect
   blob
@@ -32,6 +40,7 @@ set src {
   builtin
   bundle
   cache
+  capabilities
   captcha
   cgi
   checkin
@@ -41,20 +50,24 @@ set src {
   comformat
   configure
   content
+  cookies
   db
   delta
   deltacmd
+  deltafunc
   descendants
   diff
   diffcmd
   dispatch
   doc
   encode
+  etag
   event
   export
   file
   finfo
   foci
+  forum
   fshell
   fusefs
   glob
@@ -104,18 +117,21 @@ set src {
   purge
   rebuild
   regexp
+  repolist
   report
   rss
   schema
   search
   security_audit
   setup
+  setupuser
   sha1
   sha1hard
   sha3
   shun
   sitemap
   skins
+  smtp
   sqlcmd
   stash
   stat
@@ -138,6 +154,7 @@ set src {
   util
   verify
   vfile
+  webmail
   wiki
   wikiformat
   winfile
@@ -154,6 +171,8 @@ set src {
 set extra_files {
   diff.tcl
   markdown.md
+  wiki.wiki
+  *.js
   ../skins/*/*.txt
 }
 
@@ -162,6 +181,7 @@ set extra_files {
 set SQLITE_OPTIONS {
   -DNDEBUG=1
   -DSQLITE_THREADSAFE=0
+  -DSQLITE_DQS=0
   -DSQLITE_DEFAULT_MEMSTATUS=0
   -DSQLITE_DEFAULT_WAL_SYNCHRONOUS=1
   -DSQLITE_LIKE_DOESNT_MATCH_BLOBS
@@ -177,11 +197,13 @@ set SQLITE_OPTIONS {
   -DSQLITE_DEFAULT_FILE_FORMAT=4
   -DSQLITE_ENABLE_EXPLAIN_COMMENTS
   -DSQLITE_ENABLE_FTS4
-  -DSQLITE_ENABLE_FTS3_PARENTHESIS
   -DSQLITE_ENABLE_DBSTAT_VTAB
   -DSQLITE_ENABLE_JSON1
   -DSQLITE_ENABLE_FTS5
   -DSQLITE_ENABLE_STMTVTAB
+  -DSQLITE_HAVE_ZLIB
+  -DSQLITE_INTROSPECTION_PRAGMAS
+  -DSQLITE_ENABLE_DBPAGE_VTAB
 }
 #lappend SQLITE_OPTIONS -DSQLITE_ENABLE_FTS3=1
 #lappend SQLITE_OPTIONS -DSQLITE_ENABLE_STAT4
@@ -190,13 +212,14 @@ set SQLITE_OPTIONS {
 
 # Options used to compile the included SQLite shell.
 #
-set SHELL_OPTIONS {
+set SHELL_OPTIONS [concat $SQLITE_OPTIONS {
   -Dmain=sqlite3_shell
   -DSQLITE_SHELL_IS_UTF8=1
   -DSQLITE_OMIT_LOAD_EXTENSION=1
   -DUSE_SYSTEM_SQLITE=$(USE_SYSTEM_SQLITE)
-  -DSQLITE_SHELL_DBNAME_PROC=fossil_open
-}
+  -DSQLITE_SHELL_DBNAME_PROC=sqlcmd_get_dbname
+  -DSQLITE_SHELL_INIT_PROC=sqlcmd_init_proc
+}]
 
 # miniz (libz drop-in alternative) precompiler flags.
 #
@@ -214,6 +237,11 @@ lappend SHELL_WIN32_OPTIONS -Dsystem=fossil_system
 lappend SHELL_WIN32_OPTIONS -Dgetenv=fossil_getenv
 lappend SHELL_WIN32_OPTIONS -Dfopen=fossil_fopen
 
+# STOP HERE.
+# Unless the build procedures changes, you should not have to edit anything
+# below this line.
+#############################################################################
+
 # Name of the final application
 #
 set name fossil
@@ -229,10 +257,6 @@ proc writeln {args} {
   }
 }
 
-# STOP HERE.
-# Unless the build procedures changes, you should not have to edit anything
-# below this line.
-
 # Expand any wildcards in "extra_files"
 set new_extra_files {}
 foreach file $extra_files {
@@ -263,9 +287,10 @@ writeln {#
 # This file is included by primary Makefile.
 #
 
-XBCC = $(BCC) $(BCCFLAGS) $(CFLAGS)
-XTCC = $(TCC) -I. -I$(SRCDIR) -I$(OBJDIR) $(TCCFLAGS) $(CFLAGS)
+XBCC = $(BCC) $(BCCFLAGS)
+XTCC = $(TCC) -I. -I$(SRCDIR) -I$(OBJDIR) $(TCCFLAGS)
 
+TESTFLAGS := -quiet
 }
 writeln -nonewline "SRC ="
 foreach s [lsort $src] {
@@ -296,9 +321,9 @@ writeln [string map [list \
     <<<MINIZ_OPTIONS>>> [join $MINIZ_OPTIONS " \\\n                "]] {
 all:	$(OBJDIR) $(APPNAME)
 
-install:	$(APPNAME)
+install:	all
 	mkdir -p $(INSTALLDIR)
-	mv $(APPNAME) $(INSTALLDIR)
+	cp $(APPNAME) $(INSTALLDIR)
 
 codecheck:	$(TRANS_SRC) $(OBJDIR)/codecheck1
 	$(OBJDIR)/codecheck1 $(TRANS_SRC)
@@ -321,6 +346,9 @@ $(OBJDIR)/mkbuiltin:	$(SRCDIR)/mkbuiltin.c
 $(OBJDIR)/mkversion:	$(SRCDIR)/mkversion.c
 	$(XBCC) -o $(OBJDIR)/mkversion $(SRCDIR)/mkversion.c
 
+$(OBJDIR)/mkcss:	$(SRCDIR)/mkcss.c
+	$(XBCC) -o $(OBJDIR)/mkcss $(SRCDIR)/mkcss.c
+
 $(OBJDIR)/codecheck1:	$(SRCDIR)/codecheck1.c
 	$(XBCC) -o $(OBJDIR)/codecheck1 $(SRCDIR)/codecheck1.c
 
@@ -338,13 +366,16 @@ $(OBJDIR)/codecheck1:	$(SRCDIR)/codecheck1.c
 # the run to just those test cases.
 #
 test:	$(OBJDIR) $(APPNAME)
-	$(TCLSH) $(SRCDIR)/../test/tester.tcl $(APPNAME) -quiet $(TESTFLAGS)
+	$(TCLSH) $(SRCDIR)/../test/tester.tcl $(APPNAME) $(TESTFLAGS)
 
 $(OBJDIR)/VERSION.h:	$(SRCDIR)/../manifest.uuid $(SRCDIR)/../manifest $(SRCDIR)/../VERSION $(OBJDIR)/mkversion
 	$(OBJDIR)/mkversion $(SRCDIR)/../manifest.uuid \
 		$(SRCDIR)/../manifest \
 		$(SRCDIR)/../VERSION >$(OBJDIR)/VERSION.h
 
+$(OBJDIR)/default_css.h:	$(SRCDIR)/default_css.txt $(OBJDIR)/mkcss
+	$(OBJDIR)/mkcss $(SRCDIR)/default_css.txt $(OBJDIR)/default_css.h
+
 # Setup the options used to compile the included SQLite library.
 SQLITE_OPTIONS = <<<SQLITE_OPTIONS>>>
 
@@ -421,7 +452,7 @@ $(SRCDIR)/../manifest:
 	# noop
 
 clean:
-	rm -rf $(OBJDIR)/* $(APPNAME)
+	-rm -rf $(OBJDIR)/* $(APPNAME)
 
 }
 
@@ -433,7 +464,7 @@ foreach s [lsort $src] {
 append mhargs "\$(SRCDIR)/sqlite3.h <<<NEXT_LINE>>>"
 append mhargs "\$(SRCDIR)/th.h <<<NEXT_LINE>>>"
 #append mhargs "\$(SRCDIR)/cson_amalgamation.h <<<NEXT_LINE>>>"
-append mhargs "\$(OBJDIR)/VERSION.h"
+append mhargs "\$(OBJDIR)/VERSION.h "
 set mhargs [string map [list <<<NEXT_LINE>>> \\\n\t] $mhargs]
 writeln "\$(OBJDIR)/page_index.h: \$(TRANS_SRC) \$(OBJDIR)/mkindex"
 writeln "\t\$(OBJDIR)/mkindex \$(TRANS_SRC) >\$@\n"
@@ -441,7 +472,7 @@ writeln "\t\$(OBJDIR)/mkindex \$(TRANS_SRC) >\$@\n"
 writeln "\$(OBJDIR)/builtin_data.h: \$(OBJDIR)/mkbuiltin \$(EXTRA_FILES)"
 writeln "\t\$(OBJDIR)/mkbuiltin --prefix \$(SRCDIR)/ \$(EXTRA_FILES) >\$@\n"
 
-writeln "\$(OBJDIR)/headers:\t\$(OBJDIR)/page_index.h \$(OBJDIR)/builtin_data.h \$(OBJDIR)/makeheaders \$(OBJDIR)/VERSION.h"
+writeln "\$(OBJDIR)/headers:\t\$(OBJDIR)/page_index.h \$(OBJDIR)/builtin_data.h \$(OBJDIR)/default_css.h \$(OBJDIR)/makeheaders \$(OBJDIR)/VERSION.h"
 writeln "\t\$(OBJDIR)/makeheaders $mhargs"
 writeln "\ttouch \$(OBJDIR)/headers"
 writeln "\$(OBJDIR)/headers: Makefile"
@@ -449,6 +480,7 @@ writeln "\$(OBJDIR)/json.o \$(OBJDIR)/json_artifact.o \$(OBJDIR)/json_branch.o \
 writeln "Makefile:"
 set extra_h(dispatch) " \$(OBJDIR)/page_index.h "
 set extra_h(builtin) " \$(OBJDIR)/builtin_data.h "
+set extra_h(style) " \$(OBJDIR)/default_css.h "
 
 foreach s [lsort $src] {
   writeln "\$(OBJDIR)/${s}_.c:\t\$(SRCDIR)/$s.c \$(OBJDIR)/translate"
@@ -579,7 +611,7 @@ BCC = $(BCCEXE)
 
 #### Enable legacy treatment of mv/rm (skip checkout files)
 #
-# FOSSIL_ENABLE_LEGACY_MV_RM = 1
+FOSSIL_ENABLE_LEGACY_MV_RM = 1
 
 #### Enable TH1 scripts in embedded documentation files
 #
@@ -605,6 +637,10 @@ BCC = $(BCCEXE)
 #
 # USE_SYSTEM_SQLITE = 1
 
+#### Use POSIX memory APIs from "sys/mman.h"
+#
+# USE_MMAN_H = 1
+
 #### Use the SQLite Encryption Extension
 #
 # USE_SEE = 1
@@ -662,9 +698,9 @@ ZLIBTARGETS =
 endif
 
 #### Disable creation of the OpenSSL shared libraries.  Also, disable support
-#    for both SSLv2 and SSLv3 (i.e. thereby forcing the use of TLS).
+#    for SSLv3 (i.e. thereby forcing the use of TLS).
 #
-SSLCONFIG += no-ssl2 no-ssl3 no-shared
+SSLCONFIG += no-ssl3 no-weak-ssl-ciphers no-shared
 
 #### When using zlib, make sure that OpenSSL is configured to use the zlib
 #    that Fossil knows about (i.e. the one within the source tree).
@@ -678,7 +714,7 @@ endif
 #    to create a hard link between an "openssl-1.x" sub-directory of the
 #    Fossil source code directory and the target OpenSSL source directory.
 #
-OPENSSLDIR = $(SRCDIR)/../compat/openssl-1.0.2l
+OPENSSLDIR = $(SRCDIR)/../compat/openssl-1.1.1c
 OPENSSLINCDIR = $(OPENSSLDIR)/include
 OPENSSLLIBDIR = $(OPENSSLDIR)
 
@@ -842,6 +878,12 @@ TCC += -DFOSSIL_ENABLE_JSON=1
 RCC += -DFOSSIL_ENABLE_JSON=1
 endif
 
+# With "sys/mman.h" support
+ifdef USE_MMAN_H
+TCC += -DUSE_MMAN_H=1
+RCC += -DUSE_MMAN_H=1
+endif
+
 # With SQLite Encryption Extension support
 ifdef USE_SEE
 TCC += -DUSE_SEE=1
@@ -906,6 +948,10 @@ else
 LIB += -lkernel32 -lws2_32
 endif
 
+#### Library required for DNS lookups.
+#
+LIB += -ldnsapi
+
 #### Tcl shell for use in running the fossil test suite.  This is only
 #    used for testing.
 #
@@ -965,6 +1011,7 @@ MAKEHEADERS = $(subst /,\,$(OBJDIR)/makeheaders.exe)
 MKINDEX     = $(subst /,\,$(OBJDIR)/mkindex.exe)
 MKBUILTIN   = $(subst /,\,$(OBJDIR)/mkbuiltin.exe)
 MKVERSION   = $(subst /,\,$(OBJDIR)/mkversion.exe)
+MKCSS       = $(subst /,\,$(OBJDIR)/mkcss.exe)
 CODECHECK1  = $(subst /,\,$(OBJDIR)/codecheck1.exe)
 CAT         = type
 CP          = copy
@@ -979,6 +1026,7 @@ MAKEHEADERS = $(OBJDIR)/makeheaders.exe
 MKINDEX     = $(OBJDIR)/mkindex.exe
 MKBUILTIN   = $(OBJDIR)/mkbuiltin.exe
 MKVERSION   = $(OBJDIR)/mkversion.exe
+MKCSS       = $(OBJDIR)/mkcss.exe
 CODECHECK1  = $(OBJDIR)/codecheck1.exe
 CAT         = cat
 CP          = cp
@@ -992,7 +1040,7 @@ endif}
 writeln {
 all:	$(OBJDIR) $(APPNAME)
 
-$(OBJDIR)/fossil.o:	$(SRCDIR)/../win/fossil.rc $(OBJDIR)/VERSION.h
+$(OBJDIR)/fossil.o:	$(SRCDIR)/../win/fossil.rc $(OBJDIR)/VERSION.h $(OBJDIR)/default_css.h
 ifdef USE_WINDOWS
 	$(CAT) $(subst /,\,$(SRCDIR)\miniz.c) | $(GREP) "define MZ_VERSION" > $(subst /,\,$(OBJDIR)\minizver.h)
 	$(CP) $(subst /,\,$(SRCDIR)\..\win\fossil.rc) $(subst /,\,$(OBJDIR))
@@ -1009,10 +1057,10 @@ endif
 install:	$(OBJDIR) $(APPNAME)
 ifdef USE_WINDOWS
 	$(MKDIR) $(subst /,\,$(INSTALLDIR))
-	$(MV) $(subst /,\,$(APPNAME)) $(subst /,\,$(INSTALLDIR))
+	$(CP) $(subst /,\,$(APPNAME)) $(subst /,\,$(INSTALLDIR))
 else
 	$(MKDIR) $(INSTALLDIR)
-	$(MV) $(APPNAME) $(INSTALLDIR)
+	$(CP) $(APPNAME) $(INSTALLDIR)
 endif
 
 $(OBJDIR):
@@ -1037,6 +1085,9 @@ $(MKBUILTIN):	$(SRCDIR)/mkbuiltin.c
 $(MKVERSION): $(SRCDIR)/mkversion.c
 	$(XBCC) -o $@ $(SRCDIR)/mkversion.c
 
+$(MKCSS): $(SRCDIR)/mkcss.c
+	$(XBCC) -o $@ $(SRCDIR)/mkcss.c
+
 $(CODECHECK1):	$(SRCDIR)/codecheck1.c
 	$(XBCC) -o $@ $(SRCDIR)/codecheck1.c
 
@@ -1049,6 +1100,9 @@ test:	$(OBJDIR) $(APPNAME)
 $(OBJDIR)/VERSION.h:	$(SRCDIR)/../manifest.uuid $(SRCDIR)/../manifest $(MKVERSION)
 	$(MKVERSION) $(SRCDIR)/../manifest.uuid $(SRCDIR)/../manifest $(SRCDIR)/../VERSION >$@
 
+$(OBJDIR)/default_css.h:	$(SRCDIR)/default_css.txt $(MKCSS)
+	$(MKCSS) $(SRCDIR)/default_css.txt $@
+
 # The USE_SYSTEM_SQLITE variable may be undefined, set to 0, or set
 # to 1. If it is set to 1, then there is no need to build or link
 # the sqlite3.o object. Instead, the system SQLite will be linked
@@ -1114,6 +1168,7 @@ endif
 
 openssl:	$(BLDTARGETS)
 	cd $(OPENSSLLIBDIR);./Configure --cross-compile-prefix=$(PREFIX) $(SSLCONFIG)
+	sed -i -e 's/^PERL=C:\\.*$$/PERL=perl.exe/i' $(OPENSSLLIBDIR)/Makefile
 	$(MAKE) -C $(OPENSSLLIBDIR) PREFIX=$(PREFIX) CC=$(PREFIX)$(TCCEXE) build_libs
 
 clean-openssl:
@@ -1173,13 +1228,14 @@ writeln "\t\$(MKINDEX) \$(TRANS_SRC) >\$@\n"
 writeln "\$(OBJDIR)/builtin_data.h:\t\$(MKBUILTIN) \$(EXTRA_FILES)"
 writeln "\t\$(MKBUILTIN) --prefix \$(SRCDIR)/ \$(EXTRA_FILES) >\$@\n"
 
-writeln "\$(OBJDIR)/headers:\t\$(OBJDIR)/page_index.h \$(OBJDIR)/builtin_data.h \$(MAKEHEADERS) \$(OBJDIR)/VERSION.h"
+writeln "\$(OBJDIR)/headers:\t\$(OBJDIR)/page_index.h \$(OBJDIR)/builtin_data.h \$(OBJDIR)/default_css.h \$(MAKEHEADERS) \$(OBJDIR)/VERSION.h"
 writeln "\t\$(MAKEHEADERS) $mhargs"
 writeln "\techo Done >\$(OBJDIR)/headers\n"
 writeln "\$(OBJDIR)/headers: Makefile\n"
 writeln "Makefile:\n"
 set extra_h(main) " \$(OBJDIR)/page_index.h "
 set extra_h(builtin) " \$(OBJDIR)/builtin_data.h "
+set extra_h(style) " \$(OBJDIR)/default_css.h "
 
 foreach s [lsort $src] {
   writeln "\$(OBJDIR)/${s}_.c:\t\$(SRCDIR)/$s.c \$(TRANSLATE)"
@@ -1270,7 +1326,7 @@ SSL    =
 CFLAGS = -o
 BCC    = $(DMDIR)\bin\dmc $(CFLAGS)
 TCC    = $(DMDIR)\bin\dmc $(CFLAGS) $(DMCDEF) $(SSL) $(INCL)
-LIBS   = $(DMDIR)\extra\lib\ zlib wsock32 advapi32
+LIBS   = $(DMDIR)\extra\lib\ zlib wsock32 advapi32 dnsapi
 }
 writeln "SQLITE_OPTIONS = [join $SQLITE_OPTIONS { }]\n"
 writeln "SHELL_OPTIONS = [join $SHELL_WIN32_OPTIONS { }]\n"
@@ -1329,6 +1385,9 @@ mkbuiltin$E: $(SRCDIR)\mkbuiltin.c
 mkversion$E: $(SRCDIR)\mkversion.c
 	$(BCC) -o$@ $**
 
+mkcss$E: $(SRCDIR)\mkcss.c
+	$(BCC) -o$@ $**
+
 codecheck1$E: $(SRCDIR)\codecheck1.c
 	$(BCC) -o$@ $**
 
@@ -1350,6 +1409,9 @@ $(OBJDIR)\cson_amalgamation.h : $(SRCDIR)\cson_amalgamation.h
 VERSION.h : mkversion$E $B\manifest.uuid $B\manifest $B\VERSION
 	+$** > $@
 
+default_css.h : mkcss$E $B\src\default_css.txt
+	+$** $B\src\default_css.txt $@
+
 page_index.h: mkindex$E $(SRC)
 	+$** > $@
 
@@ -1361,7 +1423,7 @@ clean:
 	-del *.obj *_.c *.h *.map
 
 realclean:
-	-del $(APPNAME) translate$E mkindex$E makeheaders$E mkversion$E codecheck1$E mkbuiltin$E
+	-del $(APPNAME) translate$E mkindex$E makeheaders$E mkversion$E codecheck1$E mkbuiltin$E mkcss$E
 
 $(OBJDIR)\json$O : $(SRCDIR)\json_detail.h
 $(OBJDIR)\json_artifact$O : $(SRCDIR)\json_detail.h
@@ -1388,7 +1450,7 @@ foreach s [lsort $src] {
   writeln "\t+translate\$E \$** > \$@\n"
 }
 
-writeln -nonewline "headers: makeheaders\$E page_index.h builtin_data.h VERSION.h\n\t +makeheaders\$E "
+writeln -nonewline "headers: makeheaders\$E page_index.h builtin_data.h default_css.h VERSION.h\n\t +makeheaders\$E "
 foreach s [lsort $src] {
   writeln -nonewline "${s}_.c:$s.h "
 }
@@ -1470,7 +1532,7 @@ FOSSIL_ENABLE_JSON = 0
 
 # Enable legacy treatment of the mv/rm commands?
 !ifndef FOSSIL_ENABLE_LEGACY_MV_RM
-FOSSIL_ENABLE_LEGACY_MV_RM = 0
+FOSSIL_ENABLE_LEGACY_MV_RM = 1
 !endif
 
 # Enable use of miniz instead of zlib?
@@ -1509,75 +1571,39 @@ USE_SEE = 0
 !endif
 
 !if $(FOSSIL_ENABLE_SSL)!=0
-SSLDIR    = $(B)\compat\openssl-1.0.2l
-SSLINCDIR = $(SSLDIR)\inc32
+SSLDIR    = $(B)\compat\openssl-1.1.1c
+SSLINCDIR = $(SSLDIR)\include
 !if $(FOSSIL_DYNAMIC_BUILD)!=0
-SSLLIBDIR = $(SSLDIR)\out32dll
+SSLLIBDIR = $(SSLDIR)
 !else
-SSLLIBDIR = $(SSLDIR)\out32
+SSLLIBDIR = $(SSLDIR)
 !endif
 SSLLFLAGS = /nologo /opt:ref /debug
-SSLLIB    = ssleay32.lib libeay32.lib user32.lib gdi32.lib crypt32.lib
+SSLLIB    = libssl.lib libcrypto.lib user32.lib gdi32.lib crypt32.lib
 !if "$(PLATFORM)"=="amd64" || "$(PLATFORM)"=="x64"
 !message Using 'x64' platform for OpenSSL...
-# BUGBUG (OpenSSL): Using "no-ssl*" here breaks the build.
-# SSLCONFIG = VC-WIN64A no-asm no-ssl2 no-ssl3
-SSLCONFIG = VC-WIN64A no-asm
+SSLCONFIG = VC-WIN64A no-asm no-ssl3 no-weak-ssl-ciphers
 !if $(FOSSIL_DYNAMIC_BUILD)!=0
 SSLCONFIG = $(SSLCONFIG) shared
 !else
 SSLCONFIG = $(SSLCONFIG) no-shared
 !endif
-SSLSETUP  = ms\do_win64a.bat
-!if $(FOSSIL_DYNAMIC_BUILD)!=0
-SSLNMAKE  = ms\ntdll.mak all
-!else
-SSLNMAKE  = ms\nt.mak all
-!endif
-# BUGBUG (OpenSSL): Using "OPENSSL_NO_SSL*" here breaks dynamic builds.
-!if $(FOSSIL_DYNAMIC_BUILD)==0
-SSLCFLAGS = -DOPENSSL_NO_SSL2 -DOPENSSL_NO_SSL3
-!endif
 !elseif "$(PLATFORM)"=="ia64"
 !message Using 'ia64' platform for OpenSSL...
-# BUGBUG (OpenSSL): Using "no-ssl*" here breaks the build.
-# SSLCONFIG = VC-WIN64I no-asm no-ssl2 no-ssl3
-SSLCONFIG = VC-WIN64I no-asm
+SSLCONFIG = VC-WIN64I no-asm no-ssl3 no-weak-ssl-ciphers
 !if $(FOSSIL_DYNAMIC_BUILD)!=0
 SSLCONFIG = $(SSLCONFIG) shared
 !else
 SSLCONFIG = $(SSLCONFIG) no-shared
 !endif
-SSLSETUP  = ms\do_win64i.bat
-!if $(FOSSIL_DYNAMIC_BUILD)!=0
-SSLNMAKE  = ms\ntdll.mak all
-!else
-SSLNMAKE  = ms\nt.mak all
-!endif
-# BUGBUG (OpenSSL): Using "OPENSSL_NO_SSL*" here breaks dynamic builds.
-!if $(FOSSIL_DYNAMIC_BUILD)==0
-SSLCFLAGS = -DOPENSSL_NO_SSL2 -DOPENSSL_NO_SSL3
-!endif
 !else
 !message Assuming 'x86' platform for OpenSSL...
-# BUGBUG (OpenSSL): Using "no-ssl*" here breaks the build.
-# SSLCONFIG = VC-WIN32 no-asm no-ssl2 no-ssl3
-SSLCONFIG = VC-WIN32 no-asm
+SSLCONFIG = VC-WIN32 no-asm no-ssl3 no-weak-ssl-ciphers
 !if $(FOSSIL_DYNAMIC_BUILD)!=0
 SSLCONFIG = $(SSLCONFIG) shared
 !else
 SSLCONFIG = $(SSLCONFIG) no-shared
 !endif
-SSLSETUP  = ms\do_ms.bat
-!if $(FOSSIL_DYNAMIC_BUILD)!=0
-SSLNMAKE  = ms\ntdll.mak all
-!else
-SSLNMAKE  = ms\nt.mak all
-!endif
-# BUGBUG (OpenSSL): Using "OPENSSL_NO_SSL*" here breaks dynamic builds.
-!if $(FOSSIL_DYNAMIC_BUILD)==0
-SSLCFLAGS = -DOPENSSL_NO_SSL2 -DOPENSSL_NO_SSL3
-!endif
 !endif
 !endif
 
@@ -1656,7 +1682,7 @@ BCC       = $(CC) $(CFLAGS)
 TCC       = $(CC) /c $(CFLAGS) $(MSCDEF) $(INCL)
 RCC       = $(RC) /D_WIN32 /D_MSC_VER $(MSCDEF) $(INCL)
 MTC       = mt
-LIBS      = ws2_32.lib advapi32.lib
+LIBS      = ws2_32.lib advapi32.lib dnsapi.lib
 LIBDIR    =
 
 !if $(FOSSIL_DYNAMIC_BUILD)!=0
@@ -1751,6 +1777,7 @@ foreach s [lsort $extra_files] {
     writeln " \\"
     writeln -nonewline "        "
   }
+  set s [regsub -all / $s \\]
   writeln -nonewline "\$(SRCDIR)\\${s}"; incr i
 }
 writeln "\n"
@@ -1773,9 +1800,13 @@ writeln "\$(OX)\\miniz\$O \\"; incr i
 writeln "!endif"
 writeln -nonewline "        \$(OX)\\fossil.res\n\n"
 writeln [string map [list <<<NEXT_LINE>>> \\] {
-APPNAME    = $(OX)\fossil$(E)
-PDBNAME    = $(OX)\fossil$(P)
-APPTARGETS =
+!ifndef BASEAPPNAME
+BASEAPPNAME = fossil
+!endif
+
+APPNAME     = $(OX)\$(BASEAPPNAME)$(E)
+PDBNAME     = $(OX)\$(BASEAPPNAME)$(P)
+APPTARGETS  =
 
 all: $(OX) $(APPNAME)
 
@@ -1794,11 +1825,10 @@ openssl:
 	@set PATH=$(PERLDIR);$(PATH)
 !endif
 	@pushd "$(SSLDIR)" && $(PERL) Configure $(SSLCONFIG) && popd
-	@pushd "$(SSLDIR)" && call $(SSLSETUP) && popd
 !if $(FOSSIL_ENABLE_WINXP)!=0
-	@pushd "$(SSLDIR)" && $(MAKE) /f $(SSLNMAKE) "CC=cl $(SSLCFLAGS) $(XPCFLAGS)" "LFLAGS=$(SSLLFLAGS) $(XPLDFLAGS)" && popd
+	@pushd "$(SSLDIR)" && $(MAKE) "CC=cl $(XPCFLAGS)" "LFLAGS=$(XPLDFLAGS)" && popd
 !else
-	@pushd "$(SSLDIR)" && $(MAKE) /f $(SSLNMAKE) "CC=cl $(SSLCFLAGS)" && popd
+	@pushd "$(SSLDIR)" && $(MAKE) && popd
 !endif
 !endif
 
@@ -1851,6 +1881,9 @@ mkbuiltin$E: $(SRCDIR)\mkbuiltin.c
 mkversion$E: $(SRCDIR)\mkversion.c
 	$(BCC) $**
 
+mkcss$E: $(SRCDIR)\mkcss.c
+	$(BCC) $**
+
 codecheck1$E: $(SRCDIR)\codecheck1.c
 	$(BCC) $**
 
@@ -1884,9 +1917,13 @@ $(OX)\miniz$O : $(SRCDIR)\miniz.c
 
 VERSION.h : mkversion$E $B\manifest.uuid $B\manifest $B\VERSION
 	$** > $@
+
 $(OX)\cson_amalgamation$O : $(SRCDIR)\cson_amalgamation.c
 	$(TCC) /Fo$@ /c $**
 
+default_css.h: mkcss$E $(SRCDIR)\default_css.txt
+	$** $@
+
 page_index.h: mkindex$E $(SRC)
 	$** > $@
 
@@ -1894,32 +1931,34 @@ builtin_data.h:	mkbuiltin$E $(EXTRA_FILES)
 	mkbuiltin$E --prefix $(SRCDIR)/ $(EXTRA_FILES) > $@
 
 clean:
-	del $(OX)\*.obj 2>NUL
-	del *.obj 2>NUL
-	del *_.c 2>NUL
-	del *.h 2>NUL
-	del *.ilk 2>NUL
-	del *.map 2>NUL
-	del *.res 2>NUL
-	del headers 2>NUL
-	del linkopts 2>NUL
-	del vc*.pdb 2>NUL
+	-del $(OX)\*.obj 2>NUL
+	-del *.obj 2>NUL
+	-del *_.c 2>NUL
+	-del *.h 2>NUL
+	-del *.ilk 2>NUL
+	-del *.map 2>NUL
+	-del *.res 2>NUL
+	-del headers 2>NUL
+	-del linkopts 2>NUL
+	-del vc*.pdb 2>NUL
 
 realclean: clean
-	del $(APPNAME) 2>NUL
-	del $(PDBNAME) 2>NUL
-	del translate$E 2>NUL
-	del translate$P 2>NUL
-	del mkindex$E 2>NUL
-	del mkindex$P 2>NUL
-	del makeheaders$E 2>NUL
-	del makeheaders$P 2>NUL
-	del mkversion$E 2>NUL
-	del mkversion$P 2>NUL
-	del codecheck1$E 2>NUL
-	del codecheck1$P 2>NUL
-	del mkbuiltin$E 2>NUL
-	del mkbuiltin$P 2>NUL
+	-del $(APPNAME) 2>NUL
+	-del $(PDBNAME) 2>NUL
+	-del translate$E 2>NUL
+	-del translate$P 2>NUL
+	-del mkindex$E 2>NUL
+	-del mkindex$P 2>NUL
+	-del makeheaders$E 2>NUL
+	-del makeheaders$P 2>NUL
+	-del mkversion$E 2>NUL
+	-del mkversion$P 2>NUL
+	-del mkcss$E 2>NUL
+	-del mkcss$P 2>NUL
+	-del codecheck1$E 2>NUL
+	-del codecheck1$P 2>NUL
+	-del mkbuiltin$E 2>NUL
+	-del mkbuiltin$P 2>NUL
 
 $(OBJDIR)\json$O : $(SRCDIR)\json_detail.h
 $(OBJDIR)\json_artifact$O : $(SRCDIR)\json_detail.h
@@ -1947,7 +1986,7 @@ foreach s [lsort $src] {
 writeln "fossil.res : \$B\\win\\fossil.rc"
 writeln "\t\$(RCC)  /fo \$@ \$**\n"
 
-writeln "headers: makeheaders\$E page_index.h builtin_data.h VERSION.h"
+writeln "headers: makeheaders\$E page_index.h builtin_data.h default_css.h VERSION.h"
 writeln -nonewline "\tmakeheaders\$E "
 set i 0
 foreach s [lsort $src] {
@@ -2040,7 +2079,7 @@ ZLIBSRCDIR=../../zlib/
 
 # define linker command and options
 LINK=$(PellesCDir)/bin/polink.exe
-LINKFLAGS=-subsystem:console -machine:$(TARGETMACHINE_LN) /LIBPATH:$(PellesCDir)\lib\win$(TARGETEXTEND) /LIBPATH:$(PellesCDir)\lib kernel32.lib advapi32.lib delayimp$(TARGETEXTEND).lib Wsock32.lib Crtmt$(TARGETEXTEND).lib
+LINKFLAGS=-subsystem:console -machine:$(TARGETMACHINE_LN) /LIBPATH:$(PellesCDir)\lib\win$(TARGETEXTEND) /LIBPATH:$(PellesCDir)\lib kernel32.lib advapi32.lib delayimp$(TARGETEXTEND).lib Wsock32.lib dnsapi.lib Crtmt$(TARGETEXTEND).lib
 
 # define standard C-compiler and flags, used to compile
 # the fossil binary. Some special definitions follow for
@@ -2129,8 +2168,11 @@ builtin_data.h:	$(EXTRA_FILES) mkbuiltin.exe
 VERSION.h:	version.exe ..\manifest.uuid ..\manifest ..\VERSION
 	version.exe ..\manifest.uuid ..\manifest ..\VERSION  >$@
 
+default_css.h:	mkcss.exe default_css.txt
+	mkcss.exe default_css.txt $@
+
 # generate the simplified headers
-headers: makeheaders.exe page_index.h builtin_data.h VERSION.h ../src/sqlite3.h ../src/th.h VERSION.h
+headers: makeheaders.exe page_index.h builtin_data.h default_css.h VERSION.h ../src/sqlite3.h ../src/th.h
 	makeheaders.exe $(foreach ts,$(TRANSLATEDSRC),$(ts):$(ts:_.c=.h)) ../src/sqlite3.h ../src/th.h VERSION.h
 	echo Done >$@
 
@@ -2163,12 +2205,12 @@ $(APPLICATION):	$(TRANSLATEDOBJ) $(SQLITEOBJ) $(SQLITESHELLOBJ) $(THOBJ) $(ZLIBO
 
 .PHONY: clean
 clean:
-	del /F $(TRANSLATEDOBJ) $(SQLITEOBJ) $(THOBJ) $(ZLIBOBJ) $(UTILS_OBJ) version.obj
-	del /F $(TRANSLATEDSRC)
-	del /F *.h headers
-	del /F $(RESOURCE)
+	-del /F $(TRANSLATEDOBJ) $(SQLITEOBJ) $(THOBJ) $(ZLIBOBJ) $(UTILS_OBJ) version.obj
+	-del /F $(TRANSLATEDSRC)
+	-del /F *.h headers
+	-del /F $(RESOURCE)
 
 .PHONY: clobber
 clobber: clean
-	del /F *.exe
+	-del /F *.exe
 }]
