@@ -41,6 +41,73 @@ int fossil_isdate(const char *z){
   return 1;
 }
 
+/*
+** Check to see if the string might be a compact date/time that omits
+** the punctuation.  Example:  "20190327084549" instead of
+** "2019-03-27 08:45:49".  If the string is of the appropriate form,
+** then return an alternative string (in static space) that is the same
+** string with punctuation inserted.
+**
+** If the bVerifyNotAHash flag is true, then a check is made to see if
+** the string is a hash prefix and NULL is returned if it is.  If the
+** bVerifyNotAHash flag is false, then the result is determined by syntax
+** of the input string only, without reference to the artifact table.
+*/
+char *fossil_expand_datetime(const char *zIn, int bVerifyNotAHash){
+  static char zEDate[20];
+  static const char aPunct[] = { 0, 0, '-', '-', ' ', ':', ':' };
+  int n = (int)strlen(zIn);
+  int i, j;
+
+  /* Only three forms allowed:
+  **   (1)  YYYYMMDD
+  **   (2)  YYYYMMDDHHMM
+  **   (3)  YYYYMMDDHHMMSS
+  */
+  if( n!=8 && n!=12 && n!=14 ) return 0;
+
+  /* Every character must be a digit */
+  for(i=0; fossil_isdigit(zIn[i]); i++){}
+  if( i!=n ) return 0;
+
+  /* Expand the date */
+  for(i=j=0; zIn[i]; i++){
+    if( i>=4 && (i%2)==0 ){
+      zEDate[j++] = aPunct[i/2];
+    }
+    zEDate[j++] = zIn[i];
+  }
+  zEDate[j] = 0;
+
+  /* Check for reasonable date values.
+  ** Offset references:
+  **    YYYY-MM-DD HH:MM:SS
+  **    0123456789 12345678
+  */
+
+  i = atoi(zEDate);
+  if( i<1970 || i>2100 ) return 0;
+  i = atoi(zEDate+5);
+  if( i<1 || i>12 ) return 0;
+  i = atoi(zEDate+8);
+  if( i<1 || i>31 ) return 0;
+  if( n>8 ){
+    i = atoi(zEDate+11);
+    if( i>24 ) return 0;
+    i = atoi(zEDate+14);
+    if( i>60 ) return 0;
+    if( n==14 && atoi(zEDate+17)>60 ) return 0;
+  }
+
+  /* The string is not also a hash prefix */
+  if( bVerifyNotAHash ){
+    if( db_exists("SELECT 1 FROM blob WHERE uuid GLOB '%q*'",zIn) ) return 0;
+  }
+
+  /* It looks like this may be a date.  Return it with punctuation added. */
+  return zEDate;
+}
+
 /*
 ** Return the RID that is the "root" of the branch that contains
 ** check-in "rid" if inBranch==0 or the first check-in in the branch
@@ -78,7 +145,7 @@ int start_of_branch(int rid, int inBranch){
 /*
 ** Convert a symbolic name into a RID.  Acceptable forms:
 **
-**   *  artifact hash
+**   *  artifact hash (optionally enclosed in [...])
 **   *  4-character or larger prefix of a artifact
 **   *  Symbolic Name
 **   *  "tag:" + symbolic name
@@ -96,7 +163,7 @@ int start_of_branch(int rid, int inBranch){
 ** Return the RID of the matching artifact.  Or return 0 if the name does not
 ** match any known object.  Or return -1 if the name is ambiguous.
 **
-** The zType parameter specifies the type of artifact: ci, t, w, e, g.
+** The zType parameter specifies the type of artifact: ci, t, w, e, g, f.
 ** If zType is NULL or "" or "*" then any type of artifact will serve.
 ** If zType is "br" then find the first check-in of the named branch
 ** rather than the last.
@@ -104,7 +171,7 @@ int start_of_branch(int rid, int inBranch){
 ** a check-in.
 **
 ** Note that the input zTag for types "t" and "e" is the artifact hash of
-** the ticket-change or event-change artifact, not the randomly generated
+** the ticket-change or technote-change artifact, not the randomly generated
 ** hexadecimal identifier assigned to tickets and events.  Those identifiers
 ** live in a separate namespace.
 */
@@ -114,6 +181,9 @@ int symbolic_name_to_rid(const char *zTag, const char *zType){
   int nTag;
   int i;
   int startOfBranch = 0;
+  const char *zXTag;     /* zTag with optional [...] removed */
+  int nXTag;             /* Size of zXTag */
+  const char *zDate;     /* Expanded date-time string */
 
   if( zType==0 || zType[0]==0 ){
     zType = "*";
@@ -150,11 +220,13 @@ int symbolic_name_to_rid(const char *zTag, const char *zType){
 
   /* Date and times */
   if( memcmp(zTag, "date:", 5)==0 ){
+    zDate = fossil_expand_datetime(&zTag[5],0);
+    if( zDate==0 ) zDate = &zTag[5];
     rid = db_int(0,
       "SELECT objid FROM event"
       " WHERE mtime<=julianday(%Q,fromLocal()) AND type GLOB '%q'"
       " ORDER BY mtime DESC LIMIT 1",
-      &zTag[5], zType);
+      zDate, zType);
     return rid;
   }
   if( fossil_isdate(zTag) ){
@@ -230,12 +302,24 @@ int symbolic_name_to_rid(const char *zTag, const char *zType){
     return rid;
   }
 
+  /* Remove optional [...] */
+  zXTag = zTag;
+  nXTag = nTag;
+  if( zXTag[0]=='[' ){
+    zXTag++;
+    nXTag--;
+  }
+  if( nXTag>0 && zXTag[nXTag-1]==']' ){
+    nXTag--;
+  }
+
   /* artifact hash or prefix */
-  if( nTag>=4 && nTag<=HNAME_MAX && validate16(zTag, nTag) ){
+  if( nXTag>=4 && nXTag<=HNAME_MAX && validate16(zXTag, nXTag) ){
     Stmt q;
     char zUuid[HNAME_MAX+1];
-    memcpy(zUuid, zTag, nTag+1);
-    canonical16(zUuid, nTag);
+    memcpy(zUuid, zXTag, nXTag);
+    zUuid[nXTag] = 0;
+    canonical16(zUuid, nXTag);
     rid = 0;
     if( zType[0]=='*' ){
       db_prepare(&q, "SELECT rid FROM blob WHERE uuid GLOB '%q*'", zUuid);
@@ -272,6 +356,18 @@ int symbolic_name_to_rid(const char *zTag, const char *zType){
     return rid;
   }
 
+  /* Pure numeric date/time */
+  zDate = fossil_expand_datetime(zTag, 0);
+  if( zDate ){
+    rid = db_int(0,
+      "SELECT objid FROM event"
+      " WHERE mtime<=julianday(%Q,fromLocal()) AND type GLOB '%q'"
+      " ORDER BY mtime DESC LIMIT 1",
+      zDate, zType);
+    if( rid) return rid;
+  }
+
+
   /* Undocumented:  numeric tags get translated directly into the RID */
   if( memcmp(zTag, "rid:", 4)==0 ){
     zTag += 4;
@@ -591,7 +687,8 @@ void whatis_rid(int rid, int verboseFlag){
     switch( db_column_text(&q,0)[0] ){
       case 'c':  zType = "Check-in";       break;
       case 'w':  zType = "Wiki-edit";      break;
-      case 'e':  zType = "Event";          break;
+      case 'e':  zType = "Technote";       break;
+      case 'f':  zType = "Forum-post";     break;
       case 't':  zType = "Ticket-change";  break;
       case 'g':  zType = "Tag-change";     break;
       default:   zType = "Unknown";        break;
@@ -599,7 +696,7 @@ void whatis_rid(int rid, int verboseFlag){
     fossil_print("type:       %s by %s on %s\n", zType, db_column_text(&q,2),
                  db_column_text(&q, 1));
     fossil_print("comment:    ");
-    comment_print(db_column_text(&q,3), 0, 12, -1, g.comFmtFlags);
+    comment_print(db_column_text(&q,3), 0, 12, -1, get_comment_format());
   }
   db_finalize(&q);
 
@@ -621,7 +718,7 @@ void whatis_rid(int rid, int verboseFlag){
       db_column_text(&q, 3),
       db_column_text(&q, 2));
     fossil_print("            ");
-    comment_print(db_column_text(&q,4), 0, 12, -1, g.comFmtFlags);
+    comment_print(db_column_text(&q,4), 0, 12, -1, get_comment_format());
   }
   db_finalize(&q);
 
@@ -656,7 +753,7 @@ void whatis_rid(int rid, int verboseFlag){
     fossil_print("            by user %s on %s\n",
                  db_column_text(&q,2), db_column_text(&q,3));
     fossil_print("            ");
-    comment_print(db_column_text(&q,1), 0, 12, -1, g.comFmtFlags);
+    comment_print(db_column_text(&q,1), 0, 12, -1, get_comment_format());
   }
   db_finalize(&q);
 }
@@ -915,6 +1012,21 @@ void describe_artifacts(const char *zWhere){
     zWhere /*safe-for-%s*/
   );
 
+  /* Forum posts */
+  if( db_table_exists("repository","forumpost") ){
+    db_multi_exec(
+      "INSERT OR IGNORE INTO description(rid,uuid,ctime,type,summary)\n"
+      "SELECT postblob.rid, postblob.uuid, forumpost.fmtime, 'forumpost',\n"
+      "       CASE WHEN fpid=froot THEN 'forum-post '\n"
+      "            ELSE 'forum-reply-to ' END || substr(rootblob.uuid,1,14)\n"
+      "  FROM forumpost, blob AS postblob, blob AS rootblob\n"
+      " WHERE (forumpost.fpid %s)\n"
+      "   AND postblob.rid=forumpost.fpid"
+      "   AND rootblob.rid=forumpost.froot",
+      zWhere /*safe-for-%s*/
+    );
+  }
+
   /* Everything else */
   db_multi_exec(
     "INSERT OR IGNORE INTO description(rid,uuid,type,summary)\n"
@@ -1009,6 +1121,12 @@ void bloblist_page(void){
   if( !g.perm.Read ){ login_needed(g.anon.Read); return; }
   style_header("List Of Artifacts");
   style_submenu_element("250 Largest", "bigbloblist");
+  if( g.perm.Admin ){
+    style_submenu_element("Artifact Log", "rcvfromlist");
+  }
+  if( g.perm.Write ){
+    style_submenu_element("Artifact Stats", "artifact_stats");
+  }
   if( !unpubOnly && mx>n && P("s")==0 ){
     int i;
     @ <p>Select a range of artifacts to view:</p>
@@ -1079,6 +1197,13 @@ void bigbloblist_page(void){
 
   login_check_credentials();
   if( !g.perm.Read ){ login_needed(g.anon.Read); return; }
+  if( g.perm.Admin ){
+    style_submenu_element("Artifact Log", "rcvfromlist");
+  }
+  if( g.perm.Write ){
+    style_submenu_element("Artifact Stats", "artifact_stats");
+  }
+  style_submenu_element("All Artifacts", "bloblist");
   style_header("%d Largest Artifacts", n);
   db_multi_exec(
     "CREATE TEMP TABLE toshow(rid INTEGER PRIMARY KEY);"
@@ -1096,7 +1221,8 @@ void bigbloblist_page(void){
     " WHERE description.rid=blob.rid"
     " ORDER BY length(content) DESC"
   );
-  @ <table cellpadding="2" cellspacing="0" border="1" id="bigblobtab">
+  @ <table cellpadding="2" cellspacing="0" border="1" \
+  @  class='sortable' data-column-types='NnnttT' data-init-sort='0'>
   @ <thead><tr><th align="right">Size<th align="right">RID
   @ <th align="right">Delta From<th>Hash<th>Description<th>Date</tr></thead>
   @ <tbody>
@@ -1117,7 +1243,7 @@ void bigbloblist_page(void){
   }
   @ </tbody></table>
   db_finalize(&q);
-  output_table_sorting_javascript("bigblobtab", "NnnttT", -1);
+  style_table_sorter();
   style_footer();
 }
 
