@@ -23,29 +23,142 @@
 
 /*
 ** SQL code to implement the tables needed by the stash.
+**
+** Historical schema changes:
+**
+**   2019-01-19:   stash.hash and stashfile.hash columns added.  The
+**                 corresponding stash.vid and stashfile.rid columns are
+**                 retained for compatibility with older versions of
+**                 fossil but are no longer used.
+**
+**   2016-10-16:   Change the PRIMARY KEY on stashfile from (origname,stashid)
+**                 to (newname,stashid).
+**
+**   2011-09-01:   stashfile.isLink column added
+**
 */
 static const char zStashInit[] =
 @ CREATE TABLE IF NOT EXISTS localdb.stash(
 @   stashid INTEGER PRIMARY KEY,     -- Unique stash identifier
-@   vid INTEGER,                     -- The baseline checkout for this stash
+@   vid INTEGER,                     -- Legacy baseline RID value. Do not use.
+@   hash TEXT,                       -- The SHA hash for the baseline
 @   comment TEXT,                    -- Comment for this stash.  Or NULL
 @   ctime TIMESTAMP                  -- When the stash was created
 @ );
 @ CREATE TABLE IF NOT EXISTS localdb.stashfile(
 @   stashid INTEGER REFERENCES stash,  -- Stash that contains this file
-@   rid INTEGER,                       -- Baseline content in BLOB table or 0.
 @   isAdded BOOLEAN,                   -- True if this is an added file
 @   isRemoved BOOLEAN,                 -- True if this file is deleted
 @   isExec BOOLEAN,                    -- True if file is executable
 @   isLink BOOLEAN,                    -- True if file is a symlink
+@   rid INTEGER,                       -- Legacy baseline RID value. Do not use
+@   hash TEXT,                         -- Hash for baseline or NULL
 @   origname TEXT,                     -- Original filename
 @   newname TEXT,                      -- New name for file at next check-in
-@   delta BLOB,                        -- Delta from baseline. Content if rid=0
+@   delta BLOB,                        -- Delta from baseline or raw content
 @   PRIMARY KEY(newname, stashid)
 @ );
 @ INSERT OR IGNORE INTO vvar(name, value) VALUES('stash-next', 1);
 ;
 
+/*
+** Make sure the stash and stashfile tables exist and have been
+** upgraded to their latest format.  Create and upgrade the tables
+** as necessary.
+*/
+static void stash_tables_exist_and_current(void){
+  if( db_table_has_column("localdb","stashfile","hash") ){
+    /* The schema is up-to-date.  But it could be that an older version
+    ** of Fossil that does no know about the stash.hash and stashfile.hash
+    ** columns has run since the schema was updated, and added entries that
+    ** have NULL hash columns.  Check for this case, and fill in any missing
+    ** hash values.
+    */
+    if( db_int(0, "SELECT hash IS NULL FROM stash"
+                  " ORDER BY stashid DESC LIMIT 1")
+    ){
+      db_multi_exec(
+        "UPDATE stash"
+        "   SET hash=(SELECT uuid FROM blob WHERE blob.rid=stash.vid)"
+        " WHERE hash IS NULL;"
+        "UPDATE stashfile"
+        "   SET hash=(SELECT uuid FROM blob WHERE blob.rid=stashfile.rid)"
+        " WHERE hash IS NULL AND rid>0;"
+      );
+    }
+    return;
+  }
+
+  if( !db_table_exists("localdb","stashfile")
+   || !db_table_exists("localdb","stash")
+  ){
+    /* Tables do not exist.  Create them from scratch. */
+    db_multi_exec("DROP TABLE IF EXISTS localdb.stash;");
+    db_multi_exec("DROP TABLE IF EXISTS localdb.stashfile;");
+    db_multi_exec(zStashInit /*works-like:""*/);
+    return;
+  }
+
+  /* The tables exists but are not necessarily current.  Upgrade them
+  ** to the latest format.
+  **
+  ** We can assume the 2011-09-01 format that includes the stashfile.isLink
+  ** column.  The only upgrades we need to worry about the PRIMARY KEY
+  ** change on 2016-10-16 and the addition of the "hash" columns on
+  ** 2019-01-19.
+  */
+  db_multi_exec(
+    "ALTER TABLE localdb.stash RENAME TO old_stash;"
+    "ALTER TABLE localdb.stashfile RENAME TO old_stashfile;"
+  );
+  db_multi_exec(zStashInit /*works-like:""*/);
+  db_multi_exec(
+    "INSERT INTO localdb.stash(stashid,vid,hash,comment,ctime)"
+    " SELECT stashid, vid,"
+    "   (SELECT uuid FROM blob WHERE blob.rid=old_stash.vid),"
+    "   comment, ctime FROM old_stash;"
+    "DROP TABLE old_stash;"
+  );
+  db_multi_exec(
+    "INSERT INTO localdb.stashfile(stashid,isAdded,isRemoved,isExec,"
+                                  "isLink,rid,hash,origname,newname,delta)"
+    " SELECT stashid, isAdded, isRemoved, isExec, isLink, rid,"
+    "   (SELECT uuid FROM blob WHERE blob.rid=old_stashfile.rid),"
+    "   origname, newname, delta FROM old_stashfile;"
+    "DROP TABLE old_stashfile;"
+  );
+}
+
+/*
+** Update the stash.vid and stashfile.rid values after a RID renumbering
+** event.
+*/
+void stash_rid_renumbering_event(void){
+  if( !db_table_has_column("localdb","stash","hash") ){
+    /* If the stash schema was the older style that lacked hash value, then
+    ** recovery is not possible.  Save off the old data, then reset the stash
+    ** to empty. */
+    if( db_table_exists("localdb","stash") ){
+      db_multi_exec("ALTER TABLE stash RENAME TO broken_stash;");
+      fossil_print("Unrecoverable stash content stored in \"broken_stash\"\n");
+    }
+    if( db_table_exists("localdb","stashfile") ){
+      db_multi_exec("ALTER TABLE stashfile RENAME TO broken_stashfile;");
+      fossil_print("Unrecoverable stashfile content stored"
+                   " in \"broken_stashfile\"\n");
+    }
+  }else{
+    /* Reset stash.vid and stash.rid values based on hashes */
+    db_multi_exec(
+      "UPDATE stash"
+      "   SET vid=(SELECT rid FROM blob WHERE blob.uuid=stash.hash);"
+      "UPDATE stashfile"
+      "   SET rid=(SELECT rid FROM blob WHERE blob.uuid=stashfile.hash)"
+      " WHERE hash IS NOT NULL;"
+    );
+  }
+}
+
 /*
 ** Add zFName to the stash given by stashid.  zFName might be the name of a
 ** file or a directory.  If a directory, add all changed files contained
@@ -79,9 +192,10 @@ static void stash_add_file_or_dir(int stashid, int vid, const char *zFName){
   db_prepare(&q, "%s", blob_sql_text(&sql));
   blob_reset(&sql);
   db_prepare(&ins,
-     "INSERT INTO stashfile(stashid, rid, isAdded, isRemoved, isExec, isLink,"
-                           "origname, newname, delta)"
-     "VALUES(%d,:rid,:isadd,:isrm,:isexe,:islink,:orig,:new,:content)",
+     "INSERT INTO stashfile(stashid, isAdded, isRemoved, isExec, isLink, rid, "
+                           "hash, origname, newname, delta)"
+     "VALUES(%d,:isadd,:isrm,:isexe,:islink,:rid,"
+     "(SELECT uuid FROM blob WHERE rid=:rid),:orig,:new,:content)",
      stashid
   );
   while( db_step(&q)==SQLITE_ROW ){
@@ -91,7 +205,6 @@ static void stash_add_file_or_dir(int stashid, int vid, const char *zFName){
     const char *zOrig = db_column_text(&q, 5);
     char *zPath = mprintf("%s%s", g.zLocalRoot, zName);
     Blob content;
-    int isNewLink = file_wd_islink(zPath);
 
     db_bind_int(&ins, ":rid", rid);
     db_bind_int(&ins, ":isadd", rid==0);
@@ -103,11 +216,7 @@ static void stash_add_file_or_dir(int stashid, int vid, const char *zFName){
 
     if( rid==0 ){
       /* A new file */
-      if( isNewLink ){
-        blob_read_link(&content, zPath);
-      }else{
-        blob_read_from_file(&content, zPath);
-      }
+      blob_read_from_file(&content, zPath, RepoFILE);
       db_bind_blob(&ins, ":content", &content);
     }else if( deleted ){
       blob_zero(&content);
@@ -117,18 +226,14 @@ static void stash_add_file_or_dir(int stashid, int vid, const char *zFName){
       Blob orig;
       Blob disk;
 
-      if( isNewLink ){
-        blob_read_link(&disk, zPath);
-      }else{
-        blob_read_from_file(&disk, zPath);
-      }
+      blob_read_from_file(&disk, zPath, RepoFILE);
       content_get(rid, &orig);
       blob_delta_create(&orig, &disk, &content);
       blob_reset(&orig);
       blob_reset(&disk);
       db_bind_blob(&ins, ":content", &content);
     }
-    db_bind_int(&ins, ":islink", isNewLink);
+    db_bind_int(&ins, ":islink", file_islink(zPath));
     db_step(&ins);
     db_reset(&ins);
     fossil_free(zPath);
@@ -182,9 +287,9 @@ static int stash_create(void){
   vid = db_lget_int("checkout", 0);
   vfile_check_signature(vid, 0);
   db_multi_exec(
-    "INSERT INTO stash(stashid,vid,comment,ctime)"
-    "VALUES(%d,%d,%Q,julianday('now'))",
-    stashid, vid, zComment
+    "INSERT INTO stash(stashid,vid,hash,comment,ctime)"
+    "VALUES(%d,%d,(SELECT uuid FROM blob WHERE rid=%d),%Q,julianday('now'))",
+    stashid, vid, vid, zComment
   );
   if( g.argc>3 ){
     int i;
@@ -204,8 +309,8 @@ static void stash_apply(int stashid, int nConflict){
   int vid;
   Stmt q;
   db_prepare(&q,
-     "SELECT rid, isRemoved, isExec, isLink, origname, newname, delta"
-     "  FROM stashfile WHERE stashid=%d",
+     "SELECT blob.rid, isRemoved, isExec, isLink, origname, newname, delta"
+     "  FROM stashfile, blob WHERE stashid=%d AND blob.uuid=stashfile.hash",
      stashid
   );
   vid = db_lget_int("checkout",0);
@@ -227,19 +332,15 @@ static void stash_apply(int stashid, int nConflict){
       db_multi_exec("INSERT OR IGNORE INTO sfile(pathname) VALUES(%Q)", zNew);
       db_ephemeral_blob(&q, 6, &delta);
       blob_write_to_file(&delta, zNPath);
-      file_wd_setexe(zNPath, isExec);
+      file_setexe(zNPath, isExec);
     }else if( isRemoved ){
       fossil_print("DELETE %s\n", zOrig);
       file_delete(zOPath);
     }else{
       Blob a, b, out, disk;
-      int isNewLink = file_wd_islink(zOPath);
+      int isNewLink = file_islink(zOPath);
       db_ephemeral_blob(&q, 6, &delta);
-      if( isNewLink ){
-        blob_read_link(&disk, zOPath);
-      }else{
-        blob_read_from_file(&disk, zOPath);
-      }
+      blob_read_from_file(&disk, zOPath, RepoFILE);
       content_get(rid, &a);
       blob_delta_apply(&a, &delta, &b);
       if( isLink == isNewLink && blob_compare(&disk, &a)==0 ){
@@ -251,7 +352,7 @@ static void stash_apply(int stashid, int nConflict){
         }else{
           blob_write_to_file(&b, zNPath);
         }
-        file_wd_setexe(zNPath, isExec);
+        file_setexe(zNPath, isExec);
         fossil_print("UPDATE %s\n", zNew);
       }else{
         int rc;
@@ -263,7 +364,7 @@ static void stash_apply(int stashid, int nConflict){
           rc = merge_3way(&a, zOPath, &b, &out, 0);
           blob_write_to_file(&out, zNPath);
           blob_reset(&out);
-          file_wd_setexe(zNPath, isExec);
+          file_setexe(zNPath, isExec);
         }
         if( rc ){
           fossil_print("CONFLICT %s\n", zNew);
@@ -311,8 +412,8 @@ static void stash_diff(
   Blob empty;
   blob_zero(&empty);
   db_prepare(&q,
-     "SELECT rid, isRemoved, isExec, isLink, origname, newname, delta"
-     "  FROM stashfile WHERE stashid=%d",
+     "SELECT blob.rid, isRemoved, isExec, isLink, origname, newname, delta"
+     "  FROM stashfile, blob WHERE stashid=%d AND blob.uuid=stashfile.hash",
      stashid
   );
   while( db_step(&q)==SQLITE_ROW ){
@@ -345,7 +446,7 @@ static void stash_diff(
       }
     }else{
       Blob delta;
-      int isOrigLink = file_wd_islink(zOPath);
+      int isOrigLink = file_islink(zOPath);
       db_ephemeral_blob(&q, 6, &delta);
       fossil_print("CHANGED %s\n", zNew);
       if( !isOrigLink != !isLink ){
@@ -430,14 +531,14 @@ static int stash_get_id(const char *zStashId){
 **  fossil stash show|cat ?STASHID? ?DIFF-OPTIONS?
 **  fossil stash gshow|gcat ?STASHID? ?DIFF-OPTIONS?
 **
-**     Show the contents of a stash as a diff against it's baseline.
+**     Show the contents of a stash as a diff against its baseline.
 **     With gshow and gcat, gdiff-command is used instead of internal
 **     diff logic.
 **
 **  fossil stash pop
 **  fossil stash apply ?STASHID?
 **
-**     Apply STASHID or the most recently create stash to the current
+**     Apply STASHID or the most recently created stash to the current
 **     working checkout.  The "pop" command deletes that changeset from
 **     the stash after applying it but the "apply" command retains the
 **     changeset.
@@ -478,26 +579,11 @@ void stash_cmd(void){
   const char *zCmd;
   int nCmd;
   int stashid = 0;
-  int rc;
   undo_capture_command_line();
   db_must_be_within_tree();
   db_open_config(0, 0);
   db_begin_transaction();
-  db_multi_exec(zStashInit /*works-like:""*/);
-  rc = db_exists("SELECT 1 FROM sqlite_master"
-                 " WHERE name='stashfile'"
-                 "   AND sql GLOB '* PRIMARY KEY(origname, stashid)*'");
-  if( rc!=0 ){
-    db_multi_exec(
-      "CREATE TABLE localdb.stashfile_tmp AS SELECT * FROM stashfile;"
-      "DROP TABLE stashfile;"
-    );
-    db_multi_exec(zStashInit /*works-like:""*/);
-    db_multi_exec(
-      "INSERT INTO stashfile SELECT * FROM stashfile_tmp;"
-      "DROP TABLE stashfile_tmp;"
-    );
-  }
+  stash_tables_exist_and_current();
   if( g.argc<=2 ){
     zCmd = "save";
   }else{
@@ -524,6 +610,9 @@ void stash_cmd(void){
       g.argc = nFile+2;
       if( nFile==0 ) return;
     }
+    /* Make sure the stash has committed before running the revert, so that
+    ** we have a copy of the changes before deleting them. */
+    db_commit_transaction();
     g.argv[1] = "revert";
     revert_cmd();
   }else
@@ -549,8 +638,7 @@ void stash_cmd(void){
     }
     verify_all_options();
     db_prepare(&q,
-       "SELECT stashid, (SELECT uuid FROM blob WHERE rid=vid),"
-       "       comment, datetime(ctime) FROM stash"
+       "SELECT stashid, hash, comment, datetime(ctime) FROM stash"
        " ORDER BY ctime"
     );
     if( verboseFlag ){
@@ -569,7 +657,7 @@ void stash_cmd(void){
       zCom = db_column_text(&q, 2);
       if( zCom && zCom[0] ){
         fossil_print("       ");
-        comment_print(zCom, 0, 7, width, g.comFmtFlags);
+        comment_print(zCom, 0, 7, width, get_comment_format());
       }
       if( verboseFlag ){
         db_bind_int(&q2, "$id", stashid);
@@ -643,7 +731,8 @@ void stash_cmd(void){
     if( g.argc>4 ) usage("apply STASHID");
     stashid = stash_get_id(g.argc==4 ? g.argv[3] : 0);
     undo_begin();
-    vid = db_int(0, "SELECT vid FROM stash WHERE stashid=%d", stashid);
+    vid = db_int(0, "SELECT blob.rid FROM stash,blob"
+                    " WHERE stashid=%d AND blob.uuid=stash.hash", stashid);
     nConflict = update_to(vid);
     stash_apply(stashid, nConflict);
     db_multi_exec("UPDATE vfile SET mtime=0 WHERE pathname IN "
