@@ -246,12 +246,12 @@ void url_parse_local(
       i = 5;
     }
     zFile = mprintf("%s", &zUrl[i]);
-  }else if( file_isfile(zUrl) ){
+  }else if( file_isfile(zUrl, ExtFILE) ){
     pUrlData->isFile = 1;
     zFile = mprintf("%s", zUrl);
-  }else if( file_isdir(zUrl)==1 ){
+  }else if( file_isdir(zUrl, ExtFILE)==1 ){
     zFile = mprintf("%s/FOSSIL", zUrl);
-    if( file_isfile(zFile) ){
+    if( file_isfile(zFile, ExtFILE) ){
       pUrlData->isFile = 1;
     }else{
       free(zFile);
@@ -276,7 +276,7 @@ void url_parse_local(
   }else if( pUrlData->user!=0 && pUrlData->passwd==0 && (urlFlags & URL_PROMPT_PW) ){
     url_prompt_for_password_local(pUrlData);
   }else if( pUrlData->user!=0 && ( urlFlags & URL_ASK_REMEMBER_PW ) ){
-    if( isatty(fileno(stdin)) ){
+    if( isatty(fileno(stdin)) && ( urlFlags & URL_REMEMBER_PW )==0 ){
       if( save_password_prompt(pUrlData->passwd) ){
         pUrlData->flags = urlFlags |= URL_REMEMBER_PW;
       }else{
@@ -534,6 +534,7 @@ char *url_render(
   if( zName1 && zValue1 ){
     blob_appendf(&p->url, "%s%s", zSep, zName1);
     if( zValue1[0] ) blob_appendf(&p->url, "=%T", zValue1);
+    zSep = "&";
   }
   if( zName2 && zValue2 ){
     blob_appendf(&p->url, "%s%s", zSep, zName2);
