@@ -90,8 +90,8 @@ int load_vfile_from_rid(int vid){
     return 0;
   }
   db_prepare(&ins,
-    "INSERT INTO vfile(vid,isexe,islink,rid,mrid,pathname) "
-    " VALUES(:vid,:isexe,:islink,:id,:id,:name)");
+    "INSERT INTO vfile(vid,isexe,islink,rid,mrid,pathname,mhash) "
+    " VALUES(:vid,:isexe,:islink,:id,:id,:name,NULL)");
   db_prepare(&ridq, "SELECT rid,size FROM blob WHERE uuid=:uuid");
   db_bind_int(&ins, ":vid", vid);
   manifest_file_rewind(p);
@@ -204,16 +204,16 @@ void vfile_check_signature(int vid, unsigned int cksigFlags){
     oldChnged = chnged = db_column_int(&q, 4);
     oldMtime = db_column_int64(&q, 7);
     origSize = db_column_int64(&q, 6);
-    currentSize = file_wd_size(zName);
-    currentMtime = file_wd_mtime(0);
+    currentSize = file_size(zName, RepoFILE);
+    currentMtime = file_mtime(0, 0);
 #ifndef _WIN32
     origPerm = db_column_int(&q, 8);
-    currentPerm = file_wd_perm(zName);
+    currentPerm = file_perm(zName, RepoFILE);
 #endif
     if( chnged==0 && (isDeleted || rid==0) ){
       /* "fossil rm" or "fossil add" always change the file */
       chnged = 1;
-    }else if( !file_wd_isfile_or_link(0) && currentSize>=0 ){
+    }else if( !file_isfile_or_link(0) && currentSize>=0 ){
       if( cksigFlags & CKSIG_ENOTFILE ){
         fossil_warning("not an ordinary file: %s", zName);
         nErr++;
@@ -244,12 +244,12 @@ void vfile_check_signature(int vid, unsigned int cksigFlags){
       assert( origSize==currentSize );
       if( !hname_verify_file_hash(zName, zUuid, nUuid) ) chnged = 1;
     }
-    if( (cksigFlags & CKSIG_SETMTIME) && (chnged==0 || chnged==2 || chnged==4) ){
+    if( (cksigFlags & CKSIG_SETMTIME) && (chnged==0 || chnged==2 || chnged==4)){
       i64 desiredMtime;
       if( mtime_of_manifest_file(vid,rid,&desiredMtime)==0 ){
         if( currentMtime!=desiredMtime ){
           file_set_mtime(zName, desiredMtime);
-          currentMtime = file_wd_mtime(zName);
+          currentMtime = file_mtime(zName, RepoFILE);
         }
       }
     }
@@ -318,13 +318,13 @@ void vfile_to_disk(
     content_get(rid, &content);
     if( file_is_the_same(&content, zName) ){
       blob_reset(&content);
-      if( file_wd_setexe(zName, isExe) ){
+      if( file_setexe(zName, isExe) ){
         db_multi_exec("UPDATE vfile SET mtime=%lld WHERE id=%d",
-                      file_wd_mtime(zName), id);
+                      file_mtime(zName, RepoFILE), id);
       }
       continue;
     }
-    if( promptFlag && file_wd_size(zName)>=0 ){
+    if( promptFlag && file_size(zName, RepoFILE)>=0 ){
       Blob ans;
       char *zMsg;
       char cReply;
@@ -341,11 +341,11 @@ void vfile_to_disk(
       }
     }
     if( verbose ) fossil_print("%s\n", &zName[nRepos]);
-    if( file_wd_isdir(zName)==1 ){
+    if( file_isdir(zName, RepoFILE)==1 ){
       /*TODO(dchest): remove directories? */
       fossil_fatal("%s is directory, cannot overwrite", zName);
     }
-    if( file_wd_size(zName)>=0 && (isLink || file_wd_islink(0)) ){
+    if( file_size(zName, RepoFILE)>=0 && (isLink || file_islink(0)) ){
       file_delete(zName);
     }
     if( isLink ){
@@ -353,32 +353,14 @@ void vfile_to_disk(
     }else{
       blob_write_to_file(&content, zName);
     }
-    file_wd_setexe(zName, isExe);
+    file_setexe(zName, isExe);
     blob_reset(&content);
     db_multi_exec("UPDATE vfile SET mtime=%lld WHERE id=%d",
-                  file_wd_mtime(zName), id);
+                  file_mtime(zName, RepoFILE), id);
   }
   db_finalize(&q);
 }
 
-
-/*
-** Delete from the disk every file in VFILE vid.
-*/
-void vfile_unlink(int vid){
-  Stmt q;
-  db_prepare(&q, "SELECT %Q || pathname FROM vfile"
-                 " WHERE vid=%d AND mrid>0", g.zLocalRoot, vid);
-  while( db_step(&q)==SQLITE_ROW ){
-    const char *zName;
-
-    zName = db_column_text(&q, 0);
-    file_delete(zName);
-  }
-  db_finalize(&q);
-  db_multi_exec("UPDATE vfile SET mtime=NULL WHERE vid=%d AND mrid>0", vid);
-}
-
 /*
 ** Check to see if the directory named in zPath is the top of a checkout.
 ** In other words, check to see if directory pPath contains a file named
@@ -389,11 +371,11 @@ int vfile_top_of_checkout(const char *zPath){
   int fileFound = 0;
 
   zFile = mprintf("%s/_FOSSIL_", zPath);
-  fileFound = file_size(zFile)>=1024;
+  fileFound = file_size(zFile, ExtFILE)>=1024;
   fossil_free(zFile);
   if( !fileFound ){
     zFile = mprintf("%s/.fslckout", zPath);
-    fileFound = file_size(zFile)>=1024;
+    fileFound = file_size(zFile, ExtFILE)>=1024;
     fossil_free(zFile);
   }
 
@@ -404,7 +386,7 @@ int vfile_top_of_checkout(const char *zPath){
   */
   if( !fileFound ){
     zFile = mprintf("%s/.fos", zPath);
-    fileFound = file_size(zFile)>=1024;
+    fileFound = file_size(zFile, ExtFILE)>=1024;
     fossil_free(zFile);
   }
   return fileFound;
@@ -524,26 +506,26 @@ void vfile_scan(
         /* do nothing */
 #ifdef _DIRENT_HAVE_D_TYPE
       }else if( (pEntry->d_type==DT_UNKNOWN || pEntry->d_type==DT_LNK)
-          ? (file_wd_isdir(zPath)==1) : (pEntry->d_type==DT_DIR) ){
+          ? (file_isdir(zPath, RepoFILE)==1) : (pEntry->d_type==DT_DIR) ){
 #else
-      }else if( file_wd_isdir(zPath)==1 ){
+      }else if( file_isdir(zPath, RepoFILE)==1 ){
 #endif
         if( !vfile_top_of_checkout(zPath) ){
           vfile_scan(pPath, nPrefix, scanFlags, pIgnore1, pIgnore2);
         }
 #ifdef _DIRENT_HAVE_D_TYPE
       }else if( (pEntry->d_type==DT_UNKNOWN || pEntry->d_type==DT_LNK)
-          ? (file_wd_isfile_or_link(zPath)) : (pEntry->d_type==DT_REG) ){
+          ? (file_isfile_or_link(zPath)) : (pEntry->d_type==DT_REG) ){
 #else
-      }else if( file_wd_isfile_or_link(zPath) ){
+      }else if( file_isfile_or_link(zPath) ){
 #endif
         if( (scanFlags & SCAN_TEMP)==0 || is_temporary_file(zUtf8) ){
           db_bind_text(&ins, ":file", &zPath[nPrefix+1]);
           if( scanFlags & SCAN_MTIME ){
-            db_bind_int(&ins, ":mtime", file_mtime(zPath));
+            db_bind_int(&ins, ":mtime", file_mtime(zPath, RepoFILE));
           }
           if( scanFlags & SCAN_SIZE ){
-            db_bind_int(&ins, ":size", file_size(zPath));
+            db_bind_int(&ins, ":size", file_size(zPath, RepoFILE));
           }
           db_step(&ins);
           db_reset(&ins);
@@ -644,9 +626,9 @@ int vfile_dir_scan(
         /* do nothing */
 #ifdef _DIRENT_HAVE_D_TYPE
       }else if( (pEntry->d_type==DT_UNKNOWN || pEntry->d_type==DT_LNK)
-          ? (file_wd_isdir(zPath)==1) : (pEntry->d_type==DT_DIR) ){
+          ? (file_isdir(zPath, RepoFILE)==1) : (pEntry->d_type==DT_DIR) ){
 #else
-      }else if( file_wd_isdir(zPath)==1 ){
+      }else if( file_isdir(zPath, RepoFILE)==1 ){
 #endif
         if( (scanFlags & SCAN_NESTED) || !vfile_top_of_checkout(zPath) ){
           char *zSavePath = mprintf("%s", zPath);
@@ -661,9 +643,9 @@ int vfile_dir_scan(
         }
 #ifdef _DIRENT_HAVE_D_TYPE
       }else if( (pEntry->d_type==DT_UNKNOWN || pEntry->d_type==DT_LNK)
-          ? (file_wd_isfile_or_link(zPath)) : (pEntry->d_type==DT_REG) ){
+          ? (file_isfile_or_link(zPath)) : (pEntry->d_type==DT_REG) ){
 #else
-      }else if( file_wd_isfile_or_link(zPath) ){
+      }else if( file_isfile_or_link(zPath) ){
 #endif
         db_bind_text(&upd, ":file", zOrigPath);
         db_step(&upd);
@@ -728,7 +710,7 @@ void vfile_aggregate_checksum_disk(int vid, Blob *pOut){
 
     if( isSelected ){
       md5sum_step_text(zName, -1);
-      if( file_wd_islink(zFullpath) ){
+      if( file_islink(zFullpath) ){
         /* Instead of file content, use link destination path */
         Blob pathBuf;
 
@@ -788,7 +770,7 @@ char *write_blob_to_temp_file(Blob *pBlob){
     sqlite3_free(zOut);
     sqlite3_randomness(8, &r);
     zOut = sqlite3_mprintf("file-%08llx", r);
-  }while( file_size(zOut)>=0 );
+  }while( file_size(zOut, ExtFILE)>=0 );
   blob_write_to_file(pBlob, zOut);
   return zOut;
 }
@@ -817,11 +799,7 @@ void vfile_compare_repository_to_disk(int vid){
     int rid = db_column_int(&q, 2);
 
     blob_zero(&disk);
-    if( file_wd_islink(zFullpath) ){
-      rc = blob_read_link(&disk, zFullpath);
-    }else{
-      rc = blob_read_from_file(&disk, zFullpath);
-    }
+    rc = blob_read_from_file(&disk, zFullpath, RepoFILE);
     if( rc<0 ){
       fossil_print("ERROR: cannot read file [%s]\n", zFullpath);
       blob_reset(&disk);
@@ -972,3 +950,130 @@ void test_agg_cksum_cmd(void){
   printf("manifest: %s\n", blob_str(&hash));
   printf("recorded: %s\n", blob_str(&hash2));
 }
+
+/*
+** This routine recomputes certain columns of the vfile and vmerge tables
+** when the associated repository is swapped out for a clone of the same
+** project, and the blob.rid value change.  The following columns are
+** updated:
+**
+**      vmerge.merge
+**      vfile.vid
+**      vfile.rid
+**      vfile.mrid
+**
+** Also:
+**
+**      vvar.value WHERE name='checkout'
+*/
+void vfile_rid_renumbering_event(int dryRun){
+  int oldVid;
+  int newVid;
+  char *zUnresolved;
+
+  oldVid = db_lget_int("checkout", 0);
+  newVid = db_int(0, "SELECT blob.rid FROM blob, vvar"
+                     " WHERE blob.uuid=vvar.value"
+                     "   AND vvar.name='checkout-hash'");
+
+  /* The idMap table will make old RID values into new ones */
+  db_multi_exec(
+    "CREATE TEMP TABLE idMap(oldrid INTEGER PRIMARY KEY, newrid INT);\n"
+  );
+
+  /* Add the RID value for the current check-out */
+  db_multi_exec(
+    "INSERT INTO idMap(oldrid, newrid) VALUES(%d,%d)",
+    oldVid, newVid
+  );
+
+  /* Add the RID values for any other check-ins that have been merged into
+  ** the current check-out. */
+  db_multi_exec(
+    "INSERT OR IGNORE INTO idMap(oldrid, newrid)"
+    "  SELECT vmerge.merge, blob.rid FROM vmerge, blob"
+    "   WHERE blob.uuid=vmerge.mhash;"
+  );
+
+  /* Add RID values for files in the current check-out */
+  db_multi_exec(
+    "CREATE TEMP TABLE hashoffile(name TEXT PRIMARY KEY, hash TEXT)"
+    "WITHOUT ROWID;"
+
+    "INSERT INTO hashoffile(name,hash)"
+    "  SELECT filename, uuid FROM vvar, files_of_checkin(vvar.value)"
+    "   WHERE vvar.name='checkout-hash';"
+
+    "INSERT OR IGNORE INTO idMap(oldrid, newrid)"
+    "  SELECT vfile.rid, blob.rid FROM vfile, hashoffile, blob"
+    "   WHERE hashoffile.name=coalesce(vfile.origname,vfile.pathname)"
+    "     AND blob.uuid=hashoffile.hash;"
+  );
+
+  /* Add RID values for merged-in files */
+  db_multi_exec(
+    "INSERT OR IGNORE INTO idMap(oldrid, newrid)"
+    " SELECT vfile.mrid, blob.rid FROM vfile, blob"
+    "  WHERE blob.uuid=vfile.mhash;"
+  );
+  
+  if( dryRun ){
+    Stmt q;
+    db_prepare(&q, "SELECT oldrid, newrid, blob.uuid"
+                   "  FROM idMap, blob WHERE blob.rid=idMap.newrid");
+    while( db_step(&q)==SQLITE_ROW ){
+      fossil_print("%8d -> %8d  %.25s\n", 
+         db_column_int(&q,0),
+         db_column_int(&q,1),
+         db_column_text(&q,2));
+    }
+    db_finalize(&q);
+  }
+
+  /* Verify that all RID values in the VFILE table and VMERGE table have
+  ** been resolved. */
+  zUnresolved = db_text("",
+     "WITH allrid(x) AS ("
+     "  SELECT rid FROM vfile"
+     "  UNION SELECT mrid FROM vfile"
+     "  UNION SELECT merge FROM vmerge"
+     "  UNION SELECT %d"
+     ")"
+     "SELECT group_concat(x,' ') FROM allrid"
+     " WHERE x NOT IN (SELECT oldrid FROM idMap);",
+     oldVid
+  );
+  if( zUnresolved[0] ){
+    fossil_fatal("Unresolved RID values: %s\n", zUnresolved);
+  }
+
+  /* Make the changes to the VFILE and VMERGE tables */
+  if( !dryRun ){
+    db_multi_exec(
+      "UPDATE vfile"
+      "   SET rid=(SELECT newrid FROM idMap WHERE oldrid=vfile.rid)"
+      " WHERE vid=%d AND rid>0;", oldVid);
+
+    db_multi_exec(
+      "UPDATE vfile"
+      "   SET mrid=(SELECT newrid FROM idMap WHERE oldrid=vfile.mrid)"
+      " WHERE vid=%d AND mrid>0;", oldVid);
+
+    db_multi_exec(
+      "UPDATE vfile"
+      "   SET vid=%d"
+      " WHERE vid=%d", newVid, oldVid);
+
+    db_multi_exec(
+      "UPDATE vmerge"
+      "   SET merge=(SELECT newrid FROM idMap WHERE oldrid=vmerge.merge);");
+
+    db_lset_int("checkout",newVid);
+  }
+
+  /* Clear out the TEMP tables we constructed */
+  db_multi_exec(
+    "DROP TABLE idMap;"
+    "DROP TABLE hashoffile;"
+  );
+}
