@@ -31,6 +31,7 @@
 #define WIKI_BUTTONS        0x008  /* Allow sub-menu buttons */
 #define WIKI_NOBADLINKS     0x010  /* Ignore broken hyperlinks */
 #define WIKI_LINKSONLY      0x020  /* No markup.  Only decorate links */
+#define WIKI_NEWLINE        0x040  /* Honor \n - break lines at each \n */
 #endif
 
 
@@ -683,7 +684,7 @@ static int nextWikiToken(const char *z, Renderer *p, int *pTokenType){
       if( n>0 ){
         *pTokenType = TOKEN_PARAGRAPH;
         return n;
-      }else if( fossil_isspace(z[1]) ){
+      }else{
         *pTokenType = TOKEN_NEWLINE;
         return 1;
       }
@@ -1109,24 +1110,22 @@ static int is_ticket(
   int *pClosed            /* True if the ticket is closed */
 ){
   static Stmt q;
-  static int once = 1;
   int n;
   int rc;
-  char zLower[UUID_SIZE+1];
-  char zUpper[UUID_SIZE+1];
+  char zLower[HNAME_MAX+1];
+  char zUpper[HNAME_MAX+1];
   n = strlen(zTarget);
   memcpy(zLower, zTarget, n+1);
   canonical16(zLower, n+1);
   memcpy(zUpper, zLower, n+1);
   zUpper[n-1]++;
-  if( once ){
+  if( !db_static_stmt_is_init(&q) ){
     const char *zClosedExpr = db_get("ticket-closed-expr", "status='Closed'");
     db_static_prepare(&q,
       "SELECT %s FROM ticket "
       " WHERE tkt_uuid>=:lwr AND tkt_uuid<:upr",
       zClosedExpr /*safe-for-%s*/
     );
-    once = 0;
   }
   db_bind_text(&q, ":lwr", zLower);
   db_bind_text(&q, ":upr", zUpper);
@@ -1162,6 +1161,40 @@ static const char *validWikiPageName(Renderer *p, const char *zTarget){
   return 0;
 }
 
+static const char *wikiOverrideHash = 0;
+
+/*
+** Fossil-wiki hyperlinks to wiki pages should be overridden to the
+** hash value supplied.  If the value is NULL, then override is cancelled
+** and all overwrites operate normally.
+*/
+void wiki_hyperlink_override(const char *zUuid){
+  wikiOverrideHash = zUuid;
+}
+
+
+/*
+** If links to wiki page zTarget should be redirected to some historical
+** version of that page, then return the hash of the historical version.
+** If no override is required, return NULL.
+*/
+static const char *wiki_is_overridden(const char *zTarget){
+  if( wikiOverrideHash==0 ) return 0;
+  /* The override should only happen if the override version is not the
+  ** latest version of the wiki page. */
+  if( !db_exists(
+    "SELECT 1 FROM tag, blob, tagxref AS xA, tagxref AS xB "
+    " WHERE tag.tagname GLOB 'wiki-%q*'"
+    "   AND blob.uuid GLOB '%q'"
+    "   AND xA.tagid=tag.tagid AND xA.rid=blob.rid"
+    "   AND xB.tagid=tag.tagid AND xB.mtime>xA.mtime",
+    zTarget, wikiOverrideHash
+  ) ){
+    return 0;
+  }
+  return wikiOverrideHash;
+}
+
 /*
 ** Resolve a hyperlink.  The zTarget argument is the content of the [...]
 ** in the wiki.  Append to the output string whatever text is appropriate
@@ -1220,7 +1253,7 @@ static void openHyperlink(
     blob_appendf(p->pOut, "<a href=\"%h\">", zTarget);
   }else if( is_valid_hname(zTarget) ){
     int isClosed = 0;
-    if( strlen(zTarget)<=UUID_SIZE && is_ticket(zTarget, &isClosed) ){
+    if( strlen(zTarget)<=HNAME_MAX && is_ticket(zTarget, &isClosed) ){
       /* Special display processing for tickets.  Display the hyperlink
       ** as crossed out if the ticket is closed.
       */
@@ -1261,7 +1294,12 @@ static void openHyperlink(
             && db_int(0, "SELECT datetime(%Q) NOT NULL", zTarget) ){
     blob_appendf(p->pOut, "<a href=\"%R/timeline?c=%T\">", zTarget);
   }else if( (z = validWikiPageName(p, zTarget))!=0 ){
-    blob_appendf(p->pOut, "<a href=\"%R/wiki?name=%T\">", z);
+    const char *zOverride = wiki_is_overridden(zTarget);
+    if( zOverride ){
+      blob_appendf(p->pOut, "<a href=\"%R/info/%S\">", zOverride);
+    }else{
+      blob_appendf(p->pOut, "<a href=\"%R/wiki?name=%T\">", z);
+    }
   }else if( zTarget>=&zOrig[2] && !fossil_isspace(zTarget[-2]) ){
     /* Probably an array subscript in code */
     zTerm = "";
@@ -1342,7 +1380,11 @@ static void wiki_render(Renderer *p, char *z){
         break;
       }
       case TOKEN_NEWLINE: {
-        blob_append(p->pOut, "\n", 1);
+        if( p->renderFlags & WIKI_NEWLINE ){
+          blob_append(p->pOut, "<br>\n", 5);
+        }else{
+          blob_append(p->pOut, "\n", 1);
+        }
         p->state |= AT_NEWLINE;
         break;
       }
@@ -1743,7 +1785,7 @@ void test_wiki_render(void){
   verify_all_options();
   if( g.argc!=3 ) usage("FILE");
   blob_zero(&out);
-  blob_read_from_file(&in, g.argv[2]);
+  blob_read_from_file(&in, g.argv[2], ExtFILE);
   wiki_convert(&in, &out, flags);
   blob_write_to_file(&out, "-");
 }
@@ -1833,7 +1875,7 @@ void wiki_extract_links(
       case TOKEN_LINK: {
         char *zTarget;
         int i, c;
-        char zLink[42];
+        char zLink[HNAME_MAX+4];
 
         zTarget = &z[1];
         for(i=0; zTarget[i] && zTarget[i]!='|' && zTarget[i]!=']'; i++){}
@@ -2106,7 +2148,7 @@ void test_html_tidy(void){
   int i;
 
   for(i=2; i<g.argc; i++){
-    blob_read_from_file(&in, g.argv[i]);
+    blob_read_from_file(&in, g.argv[i], ExtFILE);
     blob_zero(&out);
     htmlTidy(blob_str(&in), &out);
     blob_reset(&in);
@@ -2233,7 +2275,7 @@ void test_html_to_text(void){
   int i;
 
   for(i=2; i<g.argc; i++){
-    blob_read_from_file(&in, g.argv[i]);
+    blob_read_from_file(&in, g.argv[i], ExtFILE);
     blob_zero(&out);
     html_to_plaintext(blob_str(&in), &out);
     blob_reset(&in);
