@@ -364,7 +364,7 @@ static void xfer_accept_unversioned_file(Xfer *pXfer, int isWriter){
       " WHERE name=:name"
     );
     db_bind_int(&q, ":rcvid", g.rcvid);
-  }else if( iStatus==4 ){
+  }else if( iStatus==2 ){
     db_prepare(&q, "UPDATE unversioned SET mtime=:mtime WHERE name=:name");
   }else{
     db_prepare(&q,
@@ -974,7 +974,7 @@ static int send_unclustered(Xfer *pXfer){
     );
   }else{
     db_prepare(&q,
-      "SELECT uuid FROM unclustered JOIN blob USING(rid)"
+      "SELECT uuid FROM unclustered JOIN blob USING(rid) /*scan*/"
       " WHERE NOT EXISTS(SELECT 1 FROM shun WHERE uuid=blob.uuid)"
       "   AND NOT EXISTS(SELECT 1 FROM phantom WHERE rid=blob.rid)"
       "   AND NOT EXISTS(SELECT 1 FROM private WHERE rid=blob.rid)"
@@ -1009,34 +1009,6 @@ static void send_all(Xfer *pXfer){
   db_finalize(&q);
 }
 
-/*
-** Send a single old-style config card for configuration item zName.
-**
-** This routine and the functionality it implements is scheduled for
-** removal on 2012-05-01.
-*/
-static void send_legacy_config_card(Xfer *pXfer, const char *zName){
-  if( zName[0]!='@' ){
-    Blob val;
-    blob_zero(&val);
-    db_blob(&val, "SELECT value FROM config WHERE name=%Q", zName);
-    if( blob_size(&val)>0 ){
-      blob_appendf(pXfer->pOut, "config %s %d\n", zName, blob_size(&val));
-      blob_append(pXfer->pOut, blob_buffer(&val), blob_size(&val));
-      blob_reset(&val);
-      blob_append(pXfer->pOut, "\n", 1);
-    }
-  }else{
-    Blob content;
-    blob_zero(&content);
-    configure_render_special_name(zName, &content);
-    blob_appendf(pXfer->pOut, "config %s %d\n%s\n", zName,
-                 blob_size(&content), blob_str(&content));
-    blob_reset(&content);
-  }
-}
-
-
 /*
 ** pXfer is a "pragma uv-hash HASH" card.
 **
@@ -1165,7 +1137,6 @@ void page_xfer(void){
   int isClone = 0;
   int nGimme = 0;
   int size;
-  int recvConfig = 0;
   char *zNow;
   int rc;
   const char *zScript = 0;
@@ -1268,7 +1239,7 @@ void page_xfer(void){
     if( blob_eq(&xfer.aToken[0], "uvfile") ){
       xfer_accept_unversioned_file(&xfer, g.perm.WrUnver);
       if( blob_size(&xfer.err) ){
-          cgi_reset_content();
+        cgi_reset_content();
         @ error %T(blob_str(&xfer.err))
         nErr++;
         break;
@@ -1337,7 +1308,7 @@ void page_xfer(void){
       const char *zPCode;
       zPCode = db_get("project-code", 0);
       if( zPCode==0 ){
-        fossil_panic("missing project code");
+        fossil_fatal("missing project code");
       }
       if( !blob_eq_str(&xfer.aToken[2], zPCode, -1) ){
         cgi_reset_content();
@@ -1451,12 +1422,9 @@ void page_xfer(void){
         if( zName[0]=='/' ){
           /* New style configuration transfer */
           int groupMask = configure_name_to_mask(&zName[1], 0);
-          if( !g.perm.Admin ) groupMask &= ~CONFIGSET_USER;
+          if( !g.perm.Admin ) groupMask &= ~(CONFIGSET_USER|CONFIGSET_SCRIBER);
           if( !g.perm.RdAddr ) groupMask &= ~CONFIGSET_ADDR;
           configure_send_group(xfer.pOut, groupMask, 0);
-        }else if( configure_is_exportable(zName) ){
-          /* Old style configuration transfer */
-          send_legacy_config_card(&xfer, zName);
         }
       }
     }else
@@ -1478,10 +1446,6 @@ void page_xfer(void){
         nErr++;
         break;
       }
-      if( !recvConfig && zName[0]=='@' ){
-        configure_prepare_to_receive(0);
-        recvConfig = 1;
-      }
       configure_receive(zName, &content, CONFIGSET_ALL);
       blob_reset(&content);
       blob_seek(xfer.pIn, 1, BLOB_SEEK_CUR);
@@ -1583,6 +1547,78 @@ void page_xfer(void){
         }
         uvCatalogSent = 1;
       }
+
+      /*   pragma ci-lock CHECKIN-HASH CLIENT-ID
+      **
+      ** The client wants to make non-branch commit against the check-in
+      ** identified by CHECKIN-HASH.  The server will remember this and
+      ** subsequent ci-lock request from different clients will generate
+      ** a ci-lock-fail pragma in the reply.
+      */
+      if( blob_eq(&xfer.aToken[1], "ci-lock")
+       && xfer.nToken==4
+       && blob_is_hname(&xfer.aToken[2])
+      ){
+        Stmt q;
+        sqlite3_int64 iNow = time(0);
+        const sqlite3_int64 maxAge = 3600*24; /* Locks expire after 24 hours */
+        int seenFault = 0;
+        db_prepare(&q,
+          "SELECT json_extract(value,'$.login'),"
+          "       mtime,"
+          "       json_extract(value,'$.clientid'),"
+          "       (SELECT rid FROM blob WHERE uuid=substr(name,9)),"
+          "       name"
+          " FROM config WHERE name GLOB 'ci-lock-*'"
+        );
+        while( db_step(&q)==SQLITE_ROW ){
+          int x = db_column_int(&q,3);
+          const char *zName = db_column_text(&q,4);
+          if( db_column_int64(&q,1)<iNow-maxAge || !is_a_leaf(x) ){
+            /* check-in locks expire after maxAge seconds, or when the
+            ** check-in is no longer a leaf */
+            db_multi_exec("DELETE FROM config WHERE name=%Q", zName);
+            continue;
+          }
+          if( fossil_strcmp(zName+8, blob_str(&xfer.aToken[2]))==0 ){
+            const char *zClientId = db_column_text(&q, 2);
+            const char *zLogin = db_column_text(&q,0);
+            sqlite3_int64 mtime = db_column_int64(&q, 1);
+            if( fossil_strcmp(zClientId, blob_str(&xfer.aToken[3]))!=0 ){
+              @ pragma ci-lock-fail %F(zLogin) %lld(mtime)
+            }
+            seenFault = 1;
+          }
+        }
+        db_finalize(&q);
+        if( !seenFault ){
+          db_multi_exec(
+            "REPLACE INTO config(name,value,mtime)"
+            "VALUES('ci-lock-%q',json_object('login',%Q,'clientid',%Q),now())",
+            blob_str(&xfer.aToken[2]), g.zLogin,
+            blob_str(&xfer.aToken[3])
+          );
+        }
+      }
+
+      /*   pragma ci-unlock CLIENT-ID
+      **
+      ** Remove any locks previously held by CLIENT-ID.  Clients send this
+      ** pragma with their own ID whenever they know that they no longer
+      ** have any commits pending.
+      */
+      if( blob_eq(&xfer.aToken[1], "ci-unlock")
+       && xfer.nToken==3
+       && blob_is_hname(&xfer.aToken[2])
+      ){
+        db_multi_exec(
+          "DELETE FROM config"
+          " WHERE name GLOB 'ci-lock-*'"
+          "   AND json_extract(value,'$.clientid')=%Q",
+          blob_str(&xfer.aToken[2])
+        );
+      }
+
     }else
 
     /* Unknown message
@@ -1623,9 +1659,6 @@ void page_xfer(void){
     send_unclustered(&xfer);
     if( xfer.syncPrivate ) send_private(&xfer);
   }
-  if( recvConfig ){
-    configure_finalize_receive();
-  }
   db_multi_exec("DROP TABLE onremote");
   manifest_crosslink_end(MC_PERMIT_HOOKS);
 
@@ -1666,7 +1699,7 @@ void cmd_test_xfer(void){
     usage("?MESSAGEFILE?");
   }
   blob_zero(&g.cgiIn);
-  blob_read_from_file(&g.cgiIn, g.argc==2 ? "-" : g.argv[2]);
+  blob_read_from_file(&g.cgiIn, g.argc==2 ? "-" : g.argv[2], ExtFILE);
   disableLogin = 1;
   page_xfer();
   fossil_print("%s\n", cgi_extract_content());
@@ -1695,6 +1728,8 @@ static const char zBriefFormat[] =
 #define SYNC_FROMPARENT     0x0100    /* Pull from the parent project */
 #define SYNC_UV_TRACE       0x0200    /* Describe UV activities */
 #define SYNC_UV_DRYRUN      0x0400    /* Do not actually exchange files */
+#define SYNC_IFABLE         0x0800    /* Inability to sync is not fatal */
+#define SYNC_CKIN_LOCK      0x1000    /* Lock the current check-in */
 #endif
 
 /*
@@ -1747,6 +1782,9 @@ int client_sync(
   int nUvGimmeSent = 0;   /* Number of uvgimme cards sent on this cycle */
   int nUvFileRcvd = 0;    /* Number of uvfile cards received on this cycle */
   sqlite3_int64 mtime;    /* Modification time on a UV file */
+  int autopushFailed = 0; /* Autopush following commit failed if true */
+  const char *zCkinLock;  /* Name of check-in to lock.  NULL for none */
+  const char *zClientId;  /* A unique identifier for this check-out */
 
   if( db_get_boolean("dont-push", 0) ) syncFlags &= ~SYNC_PUSH;
   if( (syncFlags & (SYNC_PUSH|SYNC_PULL|SYNC_CLONE|SYNC_UNVERSIONED))==0
@@ -1783,12 +1821,20 @@ int client_sync(
   blob_zero(&xfer.line);
   origConfigRcvMask = 0;
 
-
   /* Send the send-private pragma if we are trying to sync private data */
   if( syncFlags & SYNC_PRIVATE ){
     blob_append(&send, "pragma send-private\n", -1);
   }
 
+  /* Figure out which check-in to lock */
+  if( syncFlags & SYNC_CKIN_LOCK ){
+    int vid = db_lget_int("checkout",0);
+    zCkinLock = db_text(0, "SELECT uuid FROM blob WHERE rid=%d", vid);
+  }else{
+    zCkinLock = 0;
+  }
+  zClientId = g.localOpen ? db_lget("client-id", 0) : 0;
+
   /* When syncing unversioned files, create a TEMP table in which to store
   ** the names of files that need to be sent from client to server.
   **
@@ -1886,12 +1932,6 @@ int client_sync(
         zName = configure_next_name(configRcvMask);
         nCardSent++;
       }
-      if( (configRcvMask & (CONFIGSET_USER|CONFIGSET_TKT))!=0
-       && (configRcvMask & CONFIGSET_OLDFORMAT)!=0
-      ){
-        int overwrite = (configRcvMask & CONFIGSET_OVERWRITE)!=0;
-        configure_prepare_to_receive(overwrite);
-      }
       origConfigRcvMask = configRcvMask;
       configRcvMask = 0;
     }
@@ -1912,17 +1952,7 @@ int client_sync(
     /* Send configuration parameters being pushed */
     if( configSendMask ){
       if( zOpType==0 ) zOpType = "Push";
-      if( configSendMask & CONFIGSET_OLDFORMAT ){
-        const char *zName;
-        zName = configure_first_name(configSendMask);
-        while( zName ){
-          send_legacy_config_card(&xfer, zName);
-          zName = configure_next_name(configSendMask);
-          nCardSent++;
-        }
-      }else{
-        nCardSent += configure_send_group(xfer.pOut, configSendMask, 0);
-      }
+      nCardSent += configure_send_group(xfer.pOut, configSendMask, 0);
       configSendMask = 0;
     }
 
@@ -1967,6 +1997,18 @@ int client_sync(
       }
     }
 
+    /* Lock the current check-out */
+    if( zCkinLock ){
+      if( zClientId==0 ){
+        zClientId = db_text(0, "SELECT lower(hex(randomblob(20)))");
+        db_lset("client-id", zClientId);
+      }
+      blob_appendf(&send, "pragma ci-lock %s %s\n", zCkinLock, zClientId);
+      zCkinLock = 0;
+    }else if( zClientId ){
+      blob_appendf(&send, "pragma ci-unlock %s\n", zClientId);
+    }
+
     /* Append randomness to the end of the message.  This makes all
     ** messages unique so that that the login-card nonce will always
     ** be unique.
@@ -2082,7 +2124,7 @@ int client_sync(
 
       /*   uvfile NAME MTIME HASH SIZE FLAGS \n CONTENT
       **
-      ** Accept an unversioned file from the client.
+      ** Accept an unversioned file from the server.
       */
       if( blob_eq(&xfer.aToken[0], "uvfile") ){
         xfer_accept_unversioned_file(&xfer, 1);
@@ -2326,6 +2368,29 @@ int client_sync(
         }else if( blob_eq(&xfer.aToken[1], "uv-push-ok") ){
           uvDoPush = 1;
         }
+
+        /*    pragma ci-lock-fail  USER-HOLDING-LOCK  LOCK-TIME
+        **
+        ** The server generates this message when a "pragma ci-lock"
+        ** is attempted on a check-in for which there is an existing
+        ** lock.  USER-HOLDING-LOCK is the name of the user who originated
+        ** the lock, and LOCK-TIME is the timestamp (seconds since 1970)
+        ** when the lock was taken.
+        */
+        else if( blob_eq(&xfer.aToken[1], "ci-lock-fail") && xfer.nToken==4 ){
+          char *zUser = blob_terminate(&xfer.aToken[2]);
+          sqlite3_int64 mtime, iNow;
+          defossilize(zUser);
+          iNow = time(NULL);
+          if( blob_is_int64(&xfer.aToken[3], &mtime) && iNow>mtime ){
+            iNow = time(NULL);
+            fossil_print("\nParent check-in locked by %s %s ago\n",
+               zUser, human_readable_age((iNow+1-mtime)/86400.0));
+          }else{
+            fossil_print("\nParent check-in locked by %s\n", zUser);
+          }
+          g.ckinLockFail = fossil_strdup(zUser);
+        }
       }else
 
       /*   error MESSAGE
@@ -2338,30 +2403,22 @@ int client_sync(
       ** is returned in the reply before the error card, so second and
       ** subsequent messages should be OK.  Nevertheless, we need to ignore
       ** the error card on the first message of a clone.
+      **
+      ** Also ignore "not authorized to write" errors if this is an
+      ** autopush following a commit.
       */
       if( blob_eq(&xfer.aToken[0],"error") && xfer.nToken==2 ){
-        if( (syncFlags & SYNC_CLONE)==0 || nCycle>0 ){
-          char *zMsg = blob_terminate(&xfer.aToken[1]);
-          defossilize(zMsg);
+        char *zMsg = blob_terminate(&xfer.aToken[1]);
+        defossilize(zMsg);
+        if( (syncFlags & SYNC_IFABLE)!=0
+         && sqlite3_strlike("%not authorized to write%",zMsg,0)==0 ){
+          autopushFailed = 1;
+          nErr++;
+        }else if( (syncFlags & SYNC_CLONE)==0 || nCycle>0 ){
           fossil_force_newline();
           fossil_print("Error: %s\n", zMsg);
-          if( fossil_strcmp(zMsg, "login failed")==0 ){
-            if( nCycle<2 ){
-              g.url.passwd = 0;
-              go = 1;
-              if( g.cgiOutput==0 ){
-                g.url.flags |= URL_PROMPT_PW;
-                g.url.flags &= ~URL_PROMPTED;
-                url_prompt_for_password();
-                url_remember();
-              }
-            }else{
-              nErr++;
-            }
-          }else{
-            blob_appendf(&xfer.err, "server says: %s\n", zMsg);
-            nErr++;
-          }
+          blob_appendf(&xfer.err, "server says: %s\n", zMsg);
+          nErr++;
           break;
         }
       }else
@@ -2388,11 +2445,6 @@ int client_sync(
       blobarray_reset(xfer.aToken, xfer.nToken);
       blob_reset(&xfer.line);
     }
-    if( (configRcvMask & (CONFIGSET_USER|CONFIGSET_TKT))!=0
-     && (configRcvMask & CONFIGSET_OLDFORMAT)!=0
-    ){
-      configure_finalize_receive();
-    }
     origConfigRcvMask = 0;
     if( nCardRcvd>0 && (syncFlags & SYNC_VERBOSE) ){
       fossil_print(zValueFormat /*works-like:"%s%d%d%d%d"*/, "Received:",
@@ -2474,6 +2526,15 @@ int client_sync(
     content_enable_dephantomize(1);
     db_end_transaction(0);
   }
+  if( nErr && autopushFailed ){
+    fossil_warning(
+      "Warning: The check-in was successful and is saved locally but you\n"
+      "         are not authorized to push the changes back to the server\n"
+      "         at %s",
+      g.url.canonical
+    );
+    nErr--;
+  }
   if( (syncFlags & SYNC_CLONE)==0 && g.rcvid && fossil_any_has_fork(g.rcvid) ){
     fossil_warning("***** WARNING: a fork has occurred *****\n"
                    "use \"fossil leaves -multiple\" for more details.");
